/**
@authors: Herve Milcent AB/CO-IS
@date: 14/10/2008
@file LHCCircuitHMI.ctl

*/

/**
* @name LHCCircuit functions
This is a set of functions provided to manage the LHCCircuitHMI panel
* @{
*/

// include
#uses "unLHCCircuit_TreeOverview.ctl"
#uses "CtrlLHCCircuit_TreeOverview"
#uses "CtrlRDBAccess"
#uses "unLHCCircuit_CircuitWidget.ctl"
//@{

//---------------------------------------------------------------------------------------------------------------------------------------
/** init
*/

bool LHCCircuitHMI_init()
{
//  DebugTN("LHCCircuitHMI_init");
  g_bMaxModuleActive = TRUE;
  g_iMaxOpenedModule = LHC_CIRCUIT_MAX_OPENED_CIRCUIT;
  return unGraphicalFrame_checkLibLoaded(makeDynString("CtrlLHCCircuit_TreeOverview_getCircuitDevice", "rdbOption","unLHCCircuit_CircuitWidget_RegisterCB"));
}

//---------------------------------------------------------------------------------------------------------------------------------------
/** applicationUpdateDeviceList
*/

void LHCCircuitHMI_applicationUpdateDeviceList(string sdpe, int iCommand, string sdpe2, string sSystemName)
{
  
  dyn_string dsCircuit, dsInterlockType, dsCircuitType, dsBuilding, dsPWS;
  int iDeviceNumber;

//DebugTN("applicationUpdateDeviceList", iCommand, sSystemName);
  synchronized(g_dsTreeApplication) {
    g_m_unicosGraphicalFrame_bUpdate[sSystemName] = true;
    if(g_m_bTreeDeviceOverviewRegisteredSystemInitialized[sSystemName]) {
      if(!g_m_bTreeDeviceOverview_gettingDeviceRunning[sSystemName]) {
  //DebugN("trigger upda", sSystemName);
        g_m_bTreeDeviceOverview_gettingDeviceRunning[sSystemName] = true;
        unTreeDeviceOverview_loadAll(sSystemName, g_dsDeviceType, LHC_CIRCUIT_DEVICE_FILE);
        dsBuilding = dsPWS = makeDynString();
        unLHCCircuit_TreeOverview_getCircuitDeviceProperty(sSystemName, dsPWS, dsCircuit, dsInterlockType, dsCircuitType, dsBuilding, iDeviceNumber);
  //DebugN(dsBuilding, iDeviceNumber, dsPWS, dsCircuit, dsInterlockType, dsCircuitType);
        if(sSystemName == getSystemName()) {
          g_dsLHC_CIRCUITCircuit = dsCircuit;
          g_dsLHC_CIRCUITCircuitInterlockType = dsInterlockType;
          g_dsLHC_CIRCUITCircuitCircuitType = dsCircuitType;
        }
        g_m_dsLHC_CIRCUITCircuitBuilding[sSystemName] = dsBuilding;
        g_m_dsLHC_CIRCUITPWS[sSystemName] = dsPWS;
        g_m_dsLHC_CIRCUITDeviceNumber[sSystemName] = iDeviceNumber;

        _unLHCCircuit_TreeOverview_buildList();
        _unLHCCircuit_TreeOverview_buildDeviceList();

        unGraphicalFrame_triggerDisplay(DEVUN_ADD_COMMAND, sSystemName);
        g_m_bTreeDeviceOverview_gettingDeviceRunning[sSystemName] = false;
      }
    }
  }
//DebugTN("end applicationUpdateDeviceList", iCommand, sSystemName);
}

//---------------------------------------------------------------------------------------------------------------------------------------
/** registerTreeDeviceOverview
*/

bool LHCCircuitHMI_registerTreeDeviceOverview()
{
  int i, len;
  dyn_string dsSystemNames, dsHost, exInfo;
  dyn_int diSystemId, diPort;
  bool bRes, bConnected;
  dyn_dyn_string ddsTemp;
  
//  DebugTN("LHCCircuitHMI_registerTreeDeviceOverview");
  g_iLSADBConnect=unDB_open_LSA_Connection(g_connDatabase);
  //manage Circuit Exceptions
  g_dsCircuitException=makeDynString("RCBCH10.R3B2","RCBCH6.R3B2","RCBCH7.R3B1","RCBCH8.R3B2","RCBCH9.R3B1","RCBCV10.R3B1","RCBCV6.R3B1","RCBCV7.R3B2","RCBCV8.R3B1","RCBCV9.R3B2");
  g_dsInterlockTypeException=makeDynString(CIRCUIT_C,CIRCUIT_C,CIRCUIT_C,CIRCUIT_C,CIRCUIT_C,CIRCUIT_C,CIRCUIT_C,CIRCUIT_C,CIRCUIT_C,CIRCUIT_C);
  
/*-------------------------------------------------------------------------------------------*/
// get the list of the systems connected to the system  
/*-------------------------------------------------------------------------------------------*/
  unDistributedControl_getAllDeviceConfig(dsSystemNames, diSystemId, dsHost, diPort);
  len = dynlen(dsSystemNames);
//DebugN(dsSystemNames);
  synchronized(g_dsTreeApplication) {
    for(i=1;i<=len;i++) {
      if(strpos(dsSystemNames[i], ":") < 0)
        dsSystemNames[i] = dsSystemNames[i]+":";
      g_dsTreeDeviceOverviewRegisteredSystem[i] = dsSystemNames[i];
      g_m_bTreeDeviceOverviewRegisteredSystem[g_dsTreeDeviceOverviewRegisteredSystem[i]] = false;
      g_m_bTreeDeviceOverviewSystemConnected[g_dsTreeDeviceOverviewRegisteredSystem[i]] = false;
      g_m_ddsTreeDeviceOverviewDeviceList[g_dsTreeDeviceOverviewRegisteredSystem[i]] = ddsTemp;
      g_m_bTreeDeviceOverviewRegisteredSystemInitialized[dsSystemNames[i]] = false;
      g_m_bTreeDeviceOverview_gettingDeviceRunning[dsSystemNames[i]] = false;
      g_m_bTreeDeviceOverview_deviceListAvailable[dsSystemNames[i]] = false;
    
      g_m_sTreePVSSSystem[g_dsTreeDeviceOverviewRegisteredSystem[i]] = "";
      g_m_dsTreeFrontEnd[g_dsTreeDeviceOverviewRegisteredSystem[i]] = makeDynString();
      g_m_dsTreeFrontEndDp[g_dsTreeDeviceOverviewRegisteredSystem[i]] = makeDynString();
      g_m_dsTreeFrontEndType[g_dsTreeDeviceOverviewRegisteredSystem[i]] = makeDynString();
      g_m_dsTreeApplication[g_dsTreeDeviceOverviewRegisteredSystem[i]] = makeDynString();
      g_m_dsTreeFrontEndApplication[g_dsTreeDeviceOverviewRegisteredSystem[i]] = ddsTemp;
      g_m_dsTreeSubsystem1[g_dsTreeDeviceOverviewRegisteredSystem[i]] = makeDynString();
      g_m_dsTreeSubsystem2[g_dsTreeDeviceOverviewRegisteredSystem[i]] = makeDynString();
    }
  }
  for(i=1;i<=len;i++) {
    unDistributedControl_register("_LHCCircuitHMIHMITreeDeviceOverview_register", bRes, bConnected, dsSystemNames[i], exInfo);
//DebugN(bRes, bConnected, dsSystemNames[i]);
  }
  return true;
}

//---------------------------------------------------------------------------------------------------------------------------------------
/** waitTreeDeviceOverviewRegistration
*/

bool LHCCircuitHMI_waitTreeDeviceOverviewRegistration()
{
//  DebugTN("LHCCircuitHMI_waitTreeDeviceOverviewRegistration");
  int i, len, iDeviceNumber;
  dyn_string dsSystemNames, dsHost, exInfo;
  dyn_int diSystemId, diPort;
  dyn_string dsCircuit, dsInterlockType, dsCircuitType, dsBuilding, dsPWS;
    
/*-------------------------------------------------------------------------------------------*/
// get the list of the systems connected to the system  
/*-------------------------------------------------------------------------------------------*/
  unDistributedControl_getAllDeviceConfig(dsSystemNames, diSystemId, dsHost, diPort);
  len = dynlen(dsSystemNames);

  g_dsLHC_CIRCUITCircuit = makeDynString();
  g_dsLHC_CIRCUITCircuitInterlockType = makeDynString();
  g_dsLHC_CIRCUITCircuitCircuitType = makeDynString();
// wait until the initialization is done
  for(i=1;i<=len;i++) {
    if(strpos(dsSystemNames[i], ":") < 0)
      dsSystemNames[i] = dsSystemNames[i]+":";

    unGraphicalFrame_setLoadingText("Waiting ack ("+i+"/"+len+") from "+dsSystemNames[i]);

    while(!mappingHasKey(g_m_bTreeDeviceOverviewRegisteredSystemInitialized, dsSystemNames[i]))
      delay(1);
    while(!g_m_bTreeDeviceOverviewRegisteredSystemInitialized[dsSystemNames[i]])
      delay(1);
    g_m_dsLHC_CIRCUITCircuitBuilding[dsSystemNames[i]] = makeDynString();
    g_m_dsLHC_CIRCUITPWS[dsSystemNames[i]] = makeDynString();
    g_m_dsLHC_CIRCUITDeviceNumber[dsSystemNames[i]] = 0;
//DebugTN("registered", dsSystemNames[i], g_m_bTreeDeviceOverviewRegisteredSystemInitialized[dsSystemNames[i]], "done");

    if(g_m_bTreeDeviceOverviewSystemConnected[dsSystemNames[i]]) {
      dsBuilding = dsPWS = makeDynString();
      unLHCCircuit_TreeOverview_getCircuitDeviceProperty(dsSystemNames[i], dsPWS, dsCircuit, dsInterlockType, dsCircuitType, dsBuilding, iDeviceNumber);
//DebugN(dsSystemNames[i], dsBuilding, iDeviceNumber, dsPWS, dsCircuit, dsInterlockType, dsCircuitType);
      if(dsSystemNames[i] == getSystemName()) {
        g_dsLHC_CIRCUITCircuit = dsCircuit;
        g_dsLHC_CIRCUITCircuitInterlockType = dsInterlockType;
        g_dsLHC_CIRCUITCircuitCircuitType = dsCircuitType;
      }
      g_m_dsLHC_CIRCUITCircuitBuilding[dsSystemNames[i]] = dsBuilding;
      g_m_dsLHC_CIRCUITPWS[dsSystemNames[i]] = dsPWS;
      g_m_dsLHC_CIRCUITDeviceNumber[dsSystemNames[i]] = iDeviceNumber;
    }
    g_m_bTreeDeviceOverview_gettingDeviceRunning[dsSystemNames[i]] = false;
    unGraphicalFrame_setLoadingText("Connection ("+i+"/"+len+") to "+dsSystemNames[i]+" OK");
    unGraphicalFrame_triggerDisplay(0, dsSystemNames[i]);
//test_getSelectedDeviceList(dsSystemNames[i], g_m_ddsTreeDeviceOverviewDeviceList[dsSystemNames[i]]);
//DebugTN("end", dsSystemNames[i]);
  }
        
  _unLHCCircuit_TreeOverview_buildList();
  _unLHCCircuit_TreeOverview_buildDeviceList();
        
  unGraphicalFrame_setLoadingText("Initialisation completed");
  return true;
}

//---------------------------------------------------------------------------------------------------------------------------------------
// _LHCCircuitHMIHMITreeDeviceOverview_register
/**
Purpose: Distributed control register callback function

  @param sDpe: string, input:  the distributed control DPE of a PVSS system.
  @param bSystemConnected: bool, input: state of the system

Usage: Public

PVSS manager usage: UI (PVSS00NV) and UI extended (PVSS00NG)

Constraints:
  . PVSS version: 3.0 
  . PVSS manager type: PVSS00NG and PVSS00NV
  . operating system: WXP.
  . distributed system: yes.
*/
void _LHCCircuitHMIHMITreeDeviceOverview_register(string sDpe, bool bSystemConnected)
{
  string sSystemName;
  string sDp = dpSubStr(sDpe, DPSUB_DP);
  bool bRemote, localConnected;
  dyn_dyn_string ddsTemp;
//  dyn_string dsCircuit, dsInterlockType, dsCircuitType, dsBuilding, dsPWS;
  int iDeviceNumber;
//  dyn_string dsDomain, dsNature;
//  dyn_string dsFrontEnd, dsFrontEndDp, dsFrontEndType, dsTempApplication, dsReturn, dsReturn2, dsAllDev1, dsAllDev2;
  int i, len;
  
  synchronized(g_dsTreeApplication) {
  // get the system name and check if it is local or remote
    sSystemName = substr(sDp, strlen(c_unDistributedControl_dpName), strlen(sDp))+":";
    unDistributedControl_isRemote(bRemote, sSystemName);
  //DebugTN("unTreeDeviceOverview_register", sSystemName, mappingHasKey(g_m_bTreeDeviceOverviewRegisteredSystem, sSystemName));
    if(mappingHasKey(g_m_bTreeDeviceOverviewRegisteredSystem, sSystemName)) {
  //DebugN("has key");
      if(bRemote)
        localConnected = bSystemConnected;
      else
        localConnected = true;
      if(localConnected) { //the system is connected
    // if the system is not already connected add it: trigger the display cb
    // else do nothing
        if(!g_m_bTreeDeviceOverviewRegisteredSystem[sSystemName]) {
          g_m_bTreeDeviceOverviewRegisteredSystem[sSystemName] = true;
          g_m_bTreeDeviceOverviewSystemConnected[sSystemName] = true;
  //        g_m_ddsTreeDeviceOverviewDeviceList[sSystemName] = ddsTemp;
  //DebugTN("set connc", sSystemName);

          if(!g_m_bTreeDeviceOverviewRegisteredSystemInitialized[sSystemName]) { // startup, start thread
            g_m_bTreeDeviceOverview_gettingDeviceRunning[sSystemName] = true;
  //          startThread("unTreeDeviceOverview_loadAll", sSystemName, g_dsDeviceType, LHC_CIRCUIT_DEVICE_FILE);
            unTreeDeviceOverview_loadAll(sSystemName, g_dsDeviceType, LHC_CIRCUIT_DEVICE_FILE);
          }
        }
      }
      else { // the system is disconnected
    // if the system is connected delete it: trigger the display cb
    // else do nothing
        if(g_m_bTreeDeviceOverviewRegisteredSystem[sSystemName]) {
          g_m_bTreeDeviceOverviewRegisteredSystem[sSystemName] = false;
          g_m_bTreeDeviceOverviewSystemConnected[sSystemName] = false;
  //        g_m_ddsTreeDeviceOverviewDeviceList[sSystemName] = ddsTemp;

          g_m_sTreePVSSSystem[sSystemName] = "";
          g_m_dsTreeFrontEnd[sSystemName] = makeDynString();
          g_m_dsTreeFrontEndDp[sSystemName] = makeDynString();
          g_m_dsTreeFrontEndType[sSystemName] = makeDynString();
          g_m_dsTreeApplication[sSystemName] = makeDynString();
          g_m_dsTreeFrontEndApplication[sSystemName] = ddsTemp;
          g_m_dsTreeSubsystem1[sSystemName] = makeDynString();
          g_m_dsTreeSubsystem2[sSystemName] = makeDynString();

          _unTreeDeviceOverview_buildSubsystemList();
          _unLHCCircuit_TreeOverview_buildList();

  //DebugN("set disc", sSystemName);
          if(g_m_bTreeDeviceOverviewRegisteredSystemInitialized[sSystemName]) // startup, start thread
            unGraphicalFrame_triggerDisplay(DEVUN_DELETE_COMMAND, sSystemName);
          if(sSystemName == getSystemName()) {
            g_dsLHC_CIRCUITCircuit = makeDynString();
            g_dsLHC_CIRCUITCircuitInterlockType = makeDynString();
            g_dsLHC_CIRCUITCircuitCircuitType = makeDynString();
          }
          g_m_dsLHC_CIRCUITCircuitBuilding[sSystemName] = makeDynString();
          g_m_dsLHC_CIRCUITPWS[sSystemName] = makeDynString();
          g_m_dsLHC_CIRCUITDeviceNumber[sSystemName] = 0;
        }
        g_m_bTreeDeviceOverviewRegisteredSystemInitialized[sSystemName] = true;
      }
    }
  }
}

//---------------------------------------------------------------------------------------------------------------------------------------

//@}
