/*! \mainpage LHC_CIRCUIT PVSS component

 \section intro_sec Introduction
The LHC_CIRCUIT component is used by the LHC_CIRCUIT project.
This PVSS project is involved in the LHC operation. It allows displaying the QPS, PIC and Power Converters status in a common view based on the Electrical Circuit.
It is also the only way for the QPS operators to connect on the QPS servers.
<BR><BR><BR>

 \section install_sec Installation
The PVSS component has to be installed via the fwInstallationTool into the PVSS project.
 */

/**
@authors: Frederic BERNARD AB/CO-IS
@date: 01/12/2006
@file unLHCCircuit_GenericFunctions.ctl

Modification History:
        - Herve 27/04/2011: CIR-2 Manage correctly the QPS and nQPS data in the Synoptic A view for RB, RQF and RQD circuit
        - Herve 27/04/2011: CIR-28 powering sub-sector widget: first connection not handled correctly with the arc powering sub-sector
        - Herve 26/04/2011: CIR-23 Powering sub-sector and circuit widget: use PIC superlock for the animation
        - Herve 21/04/2011: CIR-24 Load the menu and file access configuration 
        - Herve 18/02/2011: CIR-21 QPS HDS Interlock
        - Herve 08/02/2011: CIR-20 QPS HDS Interlock
        - Herve 01/01/2011: CIR-17: DQAMGND implementation
	- Frederic 13/11/2008: complete the comments (-> doxygen)
	- Herve 25/06/2008: in unLHCCircuit_GenericFunctions_PWScloseSwitch call qpsGenericFunctions_PWScloseSwitch_Allowed
	- Herve 24/04/2008: new Circuit tree overview
	- Herve 12/03/2008: add close switch macro
	- Herve 10/10/2007: order by circuit type
	- Frederic 24/09/2007: Move functions to unLHCCircuit_PIC.ctl, unLHCCircuit_PC.ctl, unLHCCircuit_QPS.ctl
 	- Frederic 16/08/2007: Updates for the new FGC device and the new widget for PIC
	- Herve 25/04/2007: replace Devive by Device
	- Herve 24/04/2007: in unLHCCircuit_GenericFunctions_getCircuitDevice: check the dynlen of the global variables before using them
	- Herve 17/04/2006: add dll for unLHCCircuit_GenericFunctions_getCircuitType
	- Herve 16/04/2007: new function: unLHCCircuit_GenericFunctions_orderCircuitList / remove #uses "CtrlRDBAccess" --> put in LHC_CIRCUITHMI.pnl
	- Herve 10/04/2007: SOC functions: unLHCCircuit_GenericFunctions_getSOCList, unLHCCircuit_GenericFunctions_getCircuitsFromSOC
	- Herve 03/04/2007: new dll for unLHCCircuit_GenericFunctions_getCircuitDevice
  	- Herve 23/03/2007: add new functions: unLHCCircuit_GenericFunctions_getPWSCircuit, unLHCCircuit_GenericFunctions_getCircuitDevice / modify unLHCCircuit_GenericFunctions_getCircuitType
*/

/**
* @name LHCCircuit Generic Functions
This is a set of generic functions provided to manage LHCCircuit
* @{
*/

//------------------------------------------------------------------------------------------------------------------------
// unLHCCircuit_GenericFunctions_getSOCList
/**
Purpose: get the list of SOC
*/
void unLHCCircuit_GenericFunctions_getSOCList(dyn_string &dsSOC)
{
int iConnect, iCommand;
string sSQLQuery;
dyn_dyn_mixed data;

 if (g_iLSADBConnect==0)
  {
  sSQLQuery="SELECT soc_name FROM hwc_socs ORDER BY soc_name";
  iCommand=unDB_run_Command(sSQLQuery, g_connDatabase, TRUE, data);
   if (dynlen(data)>0 && iCommand==0)
    dsSOC=data[1];
  }
}

//------------------------------------------------------------------------------------------------------------------------
// unLHCCircuit_GenericFunctions_getCircuitsFromSOC
/**
Purpose: get the list of SOC
*/
void unLHCCircuit_GenericFunctions_getCircuitsFromSOC(anytype aSOC, anytype &aCircuits)
{
int iConnect, iCommand, iType, i, iLen;
string sSQLQuery;
dyn_dyn_mixed data;
dyn_string dsCircuits, dsTemp;

 if (g_iLSADBConnect==0)
  {
  iType=getType(aSOC);

  switch (iType)
   {
   case STRING_VAR:
    sSQLQuery="SELECT circuit_name FROM hwc_soc_circuits join electrical_circuits_info using (circuit_id) WHERE soc_name ='" + aSOC + "' ORDER BY circuit_name";
    iCommand=unDB_run_Command(sSQLQuery, g_connDatabase, TRUE, data);
     if (dynlen(data)>0 && iCommand==0)
      dsCircuits=data[1];
    break;

   case DYN_STRING_VAR:
    iLen=dynlen(aSOC);
    for (i=1; i<=iLen; i++)
     {    
     sSQLQuery="SELECT circuit_name FROM hwc_soc_circuits join electrical_circuits_info using (circuit_id) WHERE soc_name ='" + aSOC[i] + "' ORDER BY circuit_name";
     iCommand=unDB_run_Command(sSQLQuery, g_connDatabase, TRUE, data);
      if (dynlen(data)>0 && iCommand==0)
       {
       dsTemp=data[1];
       dynAppend(dsCircuits, dsTemp);
      }
     } 
    break;

   default:
    dsCircuits=makeDynString();
    break;
   }
  
  dynUnique(dsCircuits);
  iLen=dynlen(dsCircuits);
  for (i=iLen;i>=1;i--)
   {
   if (dsCircuits[i]=="")
    dynRemove(dsCircuits, i);
   }
  aCircuits=dsCircuits;
  }
}

//------------------------------------------------------------------------------------------------------------------------
// unLHCCircuit_GenericFunctions_getDPEtoWidgetCB
/** GenericFunctions_getDPEtoWidgetCB
*/

//one call per system name

void unLHCCircuit_GenericFunctions_getDPEtoWidgetCB(dyn_string dsDeviceDpe, dyn_string dsFrontEndDp, dyn_string &dsParameters)
{
int i, iLenDevices, iLenFE, iLen, iIndex;
string sDevice, sPLC, sAlarm, sSystem, sType, sPoweringSubSector;
dyn_string exceptionInfo, dsSystems, dsTypes;
dyn_string dsCircuitCIPC;
dyn_string dsFGCDevice;
dyn_string dsFGCCircuit;
dyn_int diFGCFault;
dyn_bool dbDType_NotConnected;
string sCircuit, sInterlockType;


 sPoweringSubSector=txtwidget.text;
 
 //LIMITATION!!!
 //ONE TYPE OF DEVICE PER SYSTEM

 iLenFE = dynlen(dsFrontEndDp);
 for(i=1;i<=iLenFE;i++)
  {
  dynAppend(dsParameters, makeDynString(dsFrontEndDp[i]+".alarm", dsFrontEndDp[i]+".enabled"));
  sSystem=unGenericDpFunctions_getSystemName(dsFrontEndDp[i]);
  dynAppend(dsSystems, sSystem);
  }

 dynUnique(dsSystems);
 if (dynlen(dsSystems)==1)
  {
  synchronized(g_mFault)
  {
  //init global per System
  if (!mappingHasKey(g_mFault, sSystem)) //not yet defined
   {
   g_mFault[sSystem]=FAULT_MASK;
   g_mConnected[sSystem]=FALSE;
   }
  }  
  
  g_bCBOk=TRUE;
  iLenDevices = dynlen(dsDeviceDpe);
  for(i=1;i<=iLenDevices;i++)
   { 
   sType=dpTypeName(dsDeviceDpe[i]);   
   dynAppend(dsTypes, sType);
   switch (sType)
    {
    case "CIPC":
     dynAppend(dsParameters, dsDeviceDpe[i] + ".PICToBIC.CMD_UPERM_B1B2_AUX"); 
     dynAppend(dsParameters, dsDeviceDpe[i] + ".PICToBIC.CMD_UPERM_B1B2_ESS");
     dynAppend(dsParameters, dsDeviceDpe[i] + ".InternalFlags.ST_SUBSEC_ABORT");
     dynAppend(dsParameters, dsDeviceDpe[i] + ".Calculated.ALL_ST_CONNECT");
     dynAppend(dsParameters, dsDeviceDpe[i] + ".Calculated.ALL_ST_ERROR_POWER");              
     dynAppend(dsCircuitCIPC, dsDeviceDpe[i] + ".statusInformation.deviceLog");
     break;
     
    case "DQGROUP":
     dynAppend(dsParameters, dsDeviceDpe[i] + ".ProcessInput.ST_OK"); 
     dynAppend(dsParameters, dsDeviceDpe[i] + ".ProcessInput.ST_PWR_PERM"); 
     dynAppend(dsParameters, dsDeviceDpe[i] + ".ProcessInput.ST_LOGGING");
     dynAppend(dsParameters, dsDeviceDpe[i] + ".ProcessInput.ST_FIP");
     dynAppend(dsParameters, dsDeviceDpe[i] + ".ProcessInput.ST_FIP_DRV");     
     break;

    case "FGC2_LHC_PC":
     dynAppend(dsParameters, dsDeviceDpe[i] + ".ProcessInput.ST_FAULTS"); 
     dynAppend(dsFGCDevice, dsDeviceDpe[i]);
     sCircuit = unLHCCircuit_GenericFunctions_getCircuit(sPoweringSubSector, dsDeviceDpe[i], sInterlockType);
     dynAppend(dsFGCCircuit, sCircuit);
     dynAppend(diFGCFault, -1);
     dynAppend(dbDType_NotConnected, unLHCCircuit_GenericFunctions_isDType_NotConnected(sPoweringSubSector, sCircuit, sInterlockType, dsDeviceDpe[i] + ".ProcessInput.ST_FAULTS"));
     break;
 
    default:
     DebugN("unLHCCircuit_GenericFunctions_getDPEtoWidgetCB(): Type Unknown: " + sType);
     break; 
    }
   }
  dynUnique(dsTypes);
  if (dynlen(dsTypes)==1){
   g_sCallBackFunction = g_sCallBackFunction_Common + sType + "_widgetCB_" + iLenDevices + "_" + iLenFE;
    synchronized(g_mFault)
    {
      switch(dsTypes[1])
      {
        case "CIPC":
          g_dsCircuitCIPC = dsCircuitCIPC;
          dynUnique(g_dsCircuitCIPC);
          dynAppend(dsParameters, dsCircuitCIPC);
//DebugN("!!!!!!!", sPoweringSubSector, g_sCallBackFunction, g_dsCircuitCIPC, dsParameters);
          break;
        case "FGC2_LHC_PC":
          g_dsFGCDevice = dsFGCDevice;
          g_dsFGCCircuit = dsFGCCircuit;
          g_diFGCFault = diFGCFault;
          g_dbDType_NotConnected = dbDType_NotConnected;
//DebugN("!!!!!!!", sPoweringSubSector, g_sCallBackFunction, g_dsFGCDevice);
          break;
      }
    }
  }
  else
   {
   dynClear(dsParameters);
   g_sCallBackFunction="";
   DebugN("unLHCCircuit_GenericFunctions_getDPEtoWidgetCB()! Several dpTypes: " + dsTypes);
   }
  }
 else
  {
  g_sCallBackFunction="";
  DebugN("unLHCCircuit_GenericFunctions_getDPEtoWidgetCB()! Data on several Systems: " + dsSystems);
  }

 unGenericDpFunctions_debugN(DEBUG_ONLY_BIT0, sPoweringSubSector, "_getDPEtoWidgetCB: " + g_sCallBackFunction, dsParameters, g_dsCircuitCIPC); 
}

//------------------------------------------------------------------------------------------------------------------------
// unLHCCircuit_GenericFunctions_getCircuit
/** return the circuit name and the interlock type of a FGC device

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param sPoweringSubSector input, the powering sub-sector
@param sFGCDp input, the FGC device DP name
@param sInterlockType output, the circuit interlock type
@return the circuit
*/
string unLHCCircuit_GenericFunctions_getCircuit(string sPoweringSubSector, string sFGCDp, string &sInterlockType)
{
  string sCircuit, sTemp;
  dyn_string dsParameters, dsTemp;
  
  unGenericDpFunctions_getParameters(sFGCDp, dsParameters);
  sTemp = dsParameters[UN_PARAMETER_DOMAIN];
  dsTemp = strsplit(sTemp, UN_CONFIG_GROUP_SEPARATOR);
  while(dynlen(dsTemp)<1)
    dynAppend(dsTemp, "");
  sCircuit = dsTemp[1];
  
  unGenericDpFunctions_getParameters(sFGCDp, dsParameters);
  sTemp = dsParameters[UN_PARAMETER_NATURE];
  dsTemp = strsplit(sTemp, UN_CONFIG_GROUP_SEPARATOR);
  while(dynlen(dsTemp)<1)
    dynAppend(dsTemp, "");
  sInterlockType = dsTemp[1];
  
  unGenericDpFunctions_debugN(DEBUG_ONLY_BIT0, sPoweringSubSector, "_getCircuit: " + sPoweringSubSector, sFGCDp, sCircuit, sInterlockType);
  return sCircuit;
}

//------------------------------------------------------------------------------------------------------------------------
// unLHCCircuit_GenericFunctions_isDType_NotConnected
/** return true if the circuit is a D type circuit and the address config is not activated

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param sPoweringSubSector input, the powering sub-sector
@param sCircuit input, the circuit name
@param sInterlockType input, the interlock type
@param sFGCDpe input, the FGC device DPE name
@return true=the circuit is a D type circuit and the address config is not activated/false=all other cases
*/
bool unLHCCircuit_GenericFunctions_isDType_NotConnected(string sPoweringSubSector, string sCircuit, string sInterlockType, string sFGCDpe)
{
  bool bResult=false;
  dyn_string exceptionInfo;
  bool configExists, isActive;
  dyn_anytype config;
  
  if(sInterlockType == CIRCUIT_D)
  {
    // get the address config
    fwPeriphAddress_get(sFGCDpe, configExists, config, isActive, exceptionInfo);
    bResult = !isActive;
    unGenericDpFunctions_debugN(DEBUG_ONLY_BIT0, sPoweringSubSector+","+sCircuit+"_isDType_NotConnected", "_isDType_NotConnected: " + sPoweringSubSector, sFGCDpe, sCircuit, sInterlockType, bResult);
  }
  return bResult;
}

//------------------------------------------------------------------------------------------------------------------------
// unLHCCircuit_GenericFunctions_getDPEtoWidgetCB_Circuits
/** GenericFunctions_getDPEtoWidgetCB_Circuits
*/
//one call per system name

void unLHCCircuit_GenericFunctions_getDPEtoWidgetCB_Circuits(dyn_string dsDeviceDpe, dyn_string dsFrontEndDp, dyn_string &dsParameters)
{
int i, iLenDevices, iLenFE, iLen, iIndex, j, iParameters, jLen, iQPS, iCircuitNumber;
string sDevice, sPLC, sAlarm, sSystem, sCircuit, sPWS, sFEApplication, sId, sType, sInterlockType, sCircuitType;
dyn_string dsDpCIPC, dsInfo, exceptionInfo, dsQPSDp, dsQPSType, dsTypes, dsSystems;
dyn_int diQPSId;

 sCircuit=txtwidget.text;
 
DebugN("unLHCCircuit_GenericFunctions_getDPEtoWidgetCB_Circuits - ??? NOT USED ???", sCircuit); 
 
 //LIMITATION!!!
 //ONE TYPE OF DEVICE PER SYSTEM

 iLenFE = dynlen(dsFrontEndDp);
 for(i=1;i<=iLenFE;i++)
  {
  dynAppend(dsParameters, makeDynString(dsFrontEndDp[i]+".alarm", dsFrontEndDp[i]+".enabled"));
  sSystem=unGenericDpFunctions_getSystemName(dsFrontEndDp[i]);
  dynAppend(dsSystems, sSystem);
  }

 dynUnique(dsSystems);
 if (dynlen(dsSystems)==1)
  {
  sSystem=dsSystems[1];
  //init global per System
  if (!mappingHasKey(g_mFault, sSystem))
   {
   g_mFault[sSystem]=FAULT_MASK;
   g_mConnected[sSystem]=FALSE;
   } 

  g_bCBOk=TRUE;
  iLenDevices = dynlen(dsDeviceDpe);
  for(i=1;i<=iLenDevices;i++)
   {
   sType=dpTypeName(dsDeviceDpe[i]);   
   dynAppend(dsTypes, sType); 

   switch (sType)
    {
    case "CIPC":
     dynClear(dsDpCIPC);     
     if (sInterlockType=="") //only one time
      unLHCCircuit_GenericFunctions_getCircuitType(sCircuit, sInterlockType, sCircuitType);
     //For the moment, only EVEN side!!! -> Only one CIPC device per circuit (!!! but we are checking the two PLC communication)
     if(iParameters==0)
      {
      unLHCCircuit_PIC_getCircuitsPICId(sSystem, dsDeviceDpe[i], sCircuit, sInterlockType, dsDpCIPC);
      iParameters=100+dynlen(dsDpCIPC); //in case of CIPC, number of dpe in CB + offset of 100 to distinguish the cb function          
      dynAppend(dsParameters, dsDpCIPC);
      }
     break;
     
    case "DQGROUP":
     //Save QPS id in the global mapping variable
     qpsGenericFunctions_GetCircuitId (sSystem, unGenericDpFunctions_getApplication(dsDeviceDpe[iLenDevices]), sCircuit, diQPSId, dsQPSDp, dsQPSType);     
     iQPS=dynlen(diQPSId);
     for (j=1; j<=iQPS; j++)
      sId=sId + diQPSId[j] + ",";
     g_mQPSId[dsDeviceDpe[i] + ".ProcessInput.ST_OK"]=sId;
               
     dynAppend(dsParameters, dsDeviceDpe[i] + ".ProcessInput.ST_OK"); 
     dynAppend(dsParameters, dsDeviceDpe[i] + ".ProcessInput.ST_PWR_PERM"); 
     dynAppend(dsParameters, dsDeviceDpe[i] + ".ProcessInput.ST_LOGGING");
     dynAppend(dsParameters, dsDeviceDpe[i] + ".ProcessInput.ST_FIP");
     dynAppend(dsParameters, dsDeviceDpe[i] + ".ProcessInput.ST_FIP_DRV");
     iParameters++;      
     break;

    case "FGC2_LHC_PC":
     iParameters++; //number of devices in CB
     dynAppend(dsParameters, dsDeviceDpe[i] + ".ProcessInput.ST_FAULTS");
     dynAppend(dsParameters, dsDeviceDpe[i] + ".ProcessInput.ST_GTWDEV");
     break;
 
    default:
     DebugN("unLHCCircuit_GenericFunctions_getDPEtoWidgetCB_Circuits(): Type Unknown: " + sType);
     break; 
    }
   }
  
  dynUnique(dsTypes);
  if (dynlen(dsTypes)==1)
   g_sCallBackFunction = g_sCallBackFunction_Common + sType + "_widgetCB_" + iParameters + "_" + iLenFE;
  else
   {
   dynClear(dsParameters);
   g_sCallBackFunction="";
   DebugN("unLHCCircuit_GenericFunctions_getDPEtoWidgetCB_Circuits()! Several dpTypes: " + dsTypes);
   }
  }
 else
  {
  DebugN("unLHCCircuit_GenericFunctions_getDPEtoWidgetCB_Circuits()! Data on several Systems: " + dsSystems);  
  }
 
 unGenericDpFunctions_debugN(DEBUG_ONLY_BIT0, sCircuit, "_getDPEtoWidgetCB Circuit", dsParameters);
 
}

//------------------------------------------------------------------------------------------------------------------------
// unLHCCircuit_GenericFunctions_widgetCB_Animation
/** GenericFunctions_widgetCB_Animation
Purpose: 

Parameters:

Usage: External function

PVSS manager usage: NG, NV
*/

void unLHCCircuit_GenericFunctions_widgetCB_Animation()
{
  bit32 value;
  int iLen, i;
  string sBodyColor, sAlarmColor, sAlarmLetter, sWarningColor, sWarningLetter, sText, sToolTipText;
  bool bBodyError, bFE, bFaultCIP, bFaultFIP, bConnected;

 sText=txtwidget.text;  //Circuit name or PW subSector name
 iLen=mappinglen(g_mConnected);  
 bConnected=unLHCCircuit_GenericFunctions_widgetCB_checkParameters(sText, iLen);
 bBodyError=!bConnected; 
 if((bConnected) && (strlen(sText)==3)) // 3 char = PW sub-sector
  bConnected = unLHCCircuit_GenericFunctions_widgetCB_setPC_state();  
   
   
     for (i=1; i<=iLen && bConnected; i++) //for each systems
      {
      bConnected=bConnected & mappingGetValue(g_mConnected, i);  
      if (bConnected)
       {
       value=mappingGetValue(g_mFault, i);
       bBodyError = bBodyError | getBit(value, BODY_ERROR);
       bFaultFIP = bFaultFIP | getBit(value, FAULT_FIPBAD);
       bFaultCIP = bFaultCIP | getBit(value, FAULT_CIPBAD);
       bFE = bFE | getBit(value, FE_BAD);
       unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, sText+" _widgetCB_Animation_Connected", "_widgetCB_Animation_Connected: " + sText + " = " + mappingGetKey(g_mFault, i), "bBodyError: " + bBodyError, "bFaultFIP: " + bFaultFIP, "bFaultCIP: " + bFaultCIP, "bFE: " + bFE, "connected: " + bConnected);   
       }
      }
    //   DebugTN(DEBUG_ONLY_BIT0, sText+" _widgetCB_Animation_Connected", "_widgetCB_Animation_Connected: " + sText , "connected: " + bConnected);   
 
     if (!bConnected)
      {
      sBodyColor="unDataNotValid";
      sAlarmColor="unLaser_Warning";
      sAlarmLetter="";
      sWarningColor="cyan";
      sWarningLetter="?";   
      }
     else
      { 
      if (bFaultFIP)
       {
       sAlarmColor="unLaser_Warning";
       sAlarmLetter="E";
       } 
      else if (bFaultCIP)
       {
       sAlarmColor="unLaser_Warning";
       sAlarmLetter="W";
       } 
      else
       {
       sAlarmColor="green";
       sAlarmLetter="";
       } 

      if (bFE)
       {
       sWarningColor="cyan";
       sWarningLetter="o";
       sBodyColor="unDataNotValid";
       }
      else 
       {
       if (bBodyError)
         {
         sBodyColor="red";
    	 sToolTipText="";	 
         }
       else
         {
         sBodyColor="green";
    	 sToolTipText="";	 
         }
   
       sWarningColor="green";
       sWarningLetter="";  
       } 
      }
 
 
    	setMultiValue(	"Body1", "backCol", sBodyColor,
    					"Body2", "foreCol",	sBodyColor,	
    					"Body1", "visible",	!bBodyError,
    					"WidgetArea", "toolTipText", sToolTipText,						
    					"AlarmText", "text", sAlarmLetter,
    					"LockBmp", "visible", false,
    					"WarningText", "foreCol", sWarningColor,
    					"WarningText", "text", sWarningLetter,
    					"WidgetArea", "visible", true,
    					"ControlStateText", "text", "",
    					"ControlStateText", "visible", true,
    					"ControlStateText", "foreCol", "yellow",
    					"modeBmp", "visible", false);
  
}

//------------------------------------------------------------------------------------------------------------------------
// unLHCCircuit_GenericFunctions_widgetCB_setPC_state
/** Set the FGC state
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui

@param dsDPE  input, list of DPE
@return true=OK/false=BAD
*/
bool unLHCCircuit_GenericFunctions_widgetCB_setPC_state()
{
  bool bReturn=false;
  int i, len;
  bool bSuperLock;
  string sSystem;
  bit32 value;
  int iFault=0;
  string sCircuit;
  
  if(g_bSuperLock_init & g_bFGC_init)
  {
    len=dynlen(g_dsFGCDevice);
    if(len >=1)
    {
      sSystem=unGenericDpFunctions_getSystemName(g_dsFGCDevice[1]);
      if (mappingHasKey(g_mFault, sSystem))
      {
        bReturn=true;
  //      DebugTN("!!!!!!!unLHCCircuit_GenericFunctions_widgetCB_setPC_state(): OK: ",g_dsFGCDevice, (string)txtwidget.text);
        for(i=1;i<=len;i++)
        {
          if(g_diFGCFault[i] != -1)
          {
            sCircuit = g_dsFGCCircuit[i];
            if(strpos(sCircuit, "RB.A") == 0) // RB circuit, add .EVEN
              sCircuit += ".EVEN";
            
            bSuperLock = picCIPC_FaceplateIsCommented(g_dsCIPCSuperLock, sCircuit);
            if(!bSuperLock) {
              iFault += g_diFGCFault[i];
            // debug if no superlock & fault!
              if(g_diFGCFault[i] != 0)
                unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "GenericFunctions_widgetCB_setPC_state FAULT:"+(string)txtwidget.text,"GenericFunctions_widgetCB_setPC_state FAULT & no SuperLock:"+(string)txtwidget.text, g_diFGCFault[i], sCircuit);
            }
          }
          else
          {
            iFault += g_diFGCFault[i];
            DebugTN("unLHCCircuit_GenericFunctions_widgetCB_setPC_state(): fgc fault not set: "+i, g_dsFGCCircuit[i], g_dsFGCDevice[i], (string)txtwidget.text);
          }
        }
        value=g_mFault[sSystem];
        //cast int to boolean with iFault
        setBit(value, BODY_ERROR, (bool)iFault);
        g_mFault[sSystem]=value;
      }
      else
        DebugTN("unLHCCircuit_GenericFunctions_widgetCB_setPC_state(): System UNKNOWN: " + sSystem, (string)txtwidget.text);
    }
    else
      DebugTN("unLHCCircuit_GenericFunctions_widgetCB_setPC_state(): bad size: " + len, mappinglen(g_mFault),(string)txtwidget.text);
    unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "GenericFunctions_widgetCB_setPC_state:"+(string)txtwidget.text,"GenericFunctions_widgetCB_setPC_state:"+(string)txtwidget.text, iFault, g_diFGCFault);
  }
  return bReturn;
}

//------------------------------------------------------------------------------------------------------------------------
// unLHCCircuit_GenericFunctions_widgetCB_main
/** GenericFunctions_widgetCB_main
Purpose: 

Parameters:

Usage: External function

PVSS manager usage: NG, NV
*/

void unLHCCircuit_GenericFunctions_widgetCB_main()
{
  mapping mFault, mConnected;
  dyn_string dsCIPCSuperLock;
  dyn_int diFGCFault;
  bool bAllEqual;

	while(TRUE)
	{		
		delay(CB_ANIMATION_DELAY);
//                DebugTN((string)txtwidget.text, "main");
                synchronized(g_mFault)
                {
                      bAllEqual=unGenericObject_compareMapping(g_mFault, mFault) && unGenericObject_compareMapping(g_mConnected, mConnected) && (g_dsCIPCSuperLock == dsCIPCSuperLock) && (diFGCFault == g_diFGCFault);
                      if (!bAllEqual)		
                      {	
                        unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "unLHCCircuit_GenericFunctions_widgetCB_main:"+(string)txtwidget.text,"unLHCCircuit_GenericFunctions_widgetCB_main:"+(string)txtwidget.text, unGenericObject_compareMapping(g_mFault, mFault), unGenericObject_compareMapping(g_mConnected, mConnected), (g_dsCIPCSuperLock == dsCIPCSuperLock), (diFGCFault == g_diFGCFault));
			unLHCCircuit_GenericFunctions_widgetCB_Animation();
                        mFault=g_mFault;
                        mConnected=g_mConnected;
                        dsCIPCSuperLock = g_dsCIPCSuperLock;
                        diFGCFault = g_diFGCFault;
                      }
                }
	}
}

//------------------------------------------------------------------------------------------------------------------------
// unLHCCircuit_GenericFunctions_getPWSCircuit
/** GenericFunctions_getPWSCircuit
Purpose: 

Parameters:

Usage: External function

PVSS manager usage: NG, NV
*/

void unLHCCircuit_GenericFunctions_getPWSCircuit(string sPWS, dyn_string &dsCircuit)
{
	dyn_string dsPWS = makeDynString(sPWS);

	unLHCCircuit_TreeOverview_getCircuit(dsPWS, makeDynString("*"), makeDynString("*"), dsCircuit);
}

//------------------------------------------------------------------------------------------------------------------------
// unLHCCircuit_GenericFunctions_getCircuitType
/** GenericFunctions_getCircuitType
Purpose: 

Parameters:

Usage: External function

PVSS manager usage: NG, NV
*/

void unLHCCircuit_GenericFunctions_getCircuitType(anytype aCircuit, anytype &aInterlockType, anytype &aCircuitType)
{
	int iType=getType(aCircuit);
	dyn_string dsCircuit, dsInterlockType, dsCircuitType;
//	int i, len, pos;
	
//DebugTN("start unLHCCircuit_GenericFunctions_getCircuitType");
	switch (iType)
	{
		case STRING_VAR:
			dsCircuit = makeDynString(aCircuit);
			break;
		case DYN_STRING_VAR:
			dsCircuit = aCircuit;
			break;
		default:
			dsCircuit = makeDynString();
			break;
	}

//DebugTN("DLL unLHCCircuit_GenericFunctions_getCircuitType", dynlen(dsCircuit));

	CtrlLHCCircuit_TreeOverview_getCircuitType(g_dsLHC_CIRCUITCircuit, g_dsLHC_CIRCUITCircuitInterlockType, 
																							g_dsLHC_CIRCUITCircuitCircuitType, 
																							dsCircuit, dsInterlockType, dsCircuitType);
	switch (iType)
	{
		case STRING_VAR:
			if(dynlen(dsInterlockType) <= 0) {
				dsInterlockType[1]="";
				dsCircuitType[1]="";
			}
			aInterlockType = dsInterlockType[1];
			aCircuitType = dsCircuitType[1];
			break;
		case DYN_STRING_VAR:
			aInterlockType = dsInterlockType;
			aCircuitType = dsCircuitType;
			break;
		default:
			dsCircuit = makeDynString();
			break;
	}
//DebugTN("end unLHCCircuit_GenericFunctions_getCircuitType");
}

//------------------------------------------------------------------------------------------------------------------------
//unLHCCircuit_GenericFunctions_getCircuitDevice
/** GenericFunctions_getCircuitDevice
Purpose: 

Parameters:

Usage: External function

PVSS manager usage: NG, NV
*/

void unLHCCircuit_GenericFunctions_getCircuitDevice(dyn_string dsCircuit, dyn_dyn_string &ddsDeviceDp, dyn_dyn_string &ddsDeviceAlias, dyn_dyn_string &ddsDeviceType)
{
dyn_string dsAppl, dsDevType, dsSubsystem1, dsAlias, dsDpName;
dyn_string dsRetAppl, dsRetDevType, dsRetAlias, dsRetDpName;
int i, len;
string sSystemName;
dyn_dyn_string ddsEmpty;
		
	ddsDeviceDp = ddsDeviceAlias = ddsDeviceType = ddsEmpty;
	len = mappinglen(g_m_ddsTreeDeviceOverviewDeviceList);
  // loop over all defined and connected system
	for(i=1;i<=len;i++)
    {
		sSystemName = mappingGetKey(g_m_ddsTreeDeviceOverviewDeviceList, i);
		if(g_m_bTreeDeviceOverviewSystemConnected[sSystemName])
      {
			if(dynlen(g_m_ddsTreeDeviceOverviewDeviceList[sSystemName]) >= TREE_DEVICE_SUBSYSTEM2)
        {
        //DebugTN("unLHCCircuit_GenericFunctions_getCircuitDevice, CtrlLHCCircuit_TreeOverview_getSelectedDeviceList", sSystemName, dynlen(g_m_ddsTreeDeviceOverviewDeviceList[sSystemName]));

				dsDpName = g_m_ddsTreeDeviceOverviewDeviceList[sSystemName][TREE_DEVICE_DPNAME];
				dsAlias = g_m_ddsTreeDeviceOverviewDeviceList[sSystemName][TREE_DEVICE_ALIAS];
				dsAppl = g_m_ddsTreeDeviceOverviewDeviceList[sSystemName][TREE_DEVICE_FEAPPLICATION];
				dsDevType = g_m_ddsTreeDeviceOverviewDeviceList[sSystemName][TREE_DEVICE_TYPE];
				dsSubsystem1 = g_m_ddsTreeDeviceOverviewDeviceList[sSystemName][TREE_DEVICE_SUBSYSTEM1];

				CtrlLHCCircuit_TreeOverview_getCircuitDevice( dsCircuit,  
																											dsDpName, dsAlias, dsDevType, dsSubsystem1, 
																											ddsDeviceDp, ddsDeviceAlias, ddsDeviceType);                                
			}
		}
	}
}

//--------------------------------------------------------------------------------------------------------------------------------
// unLHCCircuit_GenericFunctions_orderCircuitList
/**
Purpose: Oder the list of circuit by interlock type and circuit type
	
	@param dsCircuit: dyn_string, input/output: the list of circuit.
*/
void unLHCCircuit_GenericFunctions_orderCircuitList(dyn_string &dsCircuit)
{
	dyn_string dsResult;
	dyn_string dsCircuitTypeOrder = makeDynString("MAINDIPOLE", "MAINQUAD", "IT", "IPQ", "600AEE", "600ANOEE", "600ANOEECR", "IPD", "80120A", "60A");
//DebugTN("start unLHCCircuit_GenericFunctions_orderCircuitList", dsCircuit, dynUnique(dsCircuit));
 CtrlLHCCircuit_TreeOverview_orderCircuitListByCircuitType(dsCircuitTypeOrder, g_dsLHC_CIRCUITCircuit, 
                        g_dsLHC_CIRCUITCircuitInterlockType, g_dsLHC_CIRCUITCircuitCircuitType,
                        dsCircuit, dsResult);
 dsCircuit = dsResult;
//DebugTN("end unLHCCircuit_GenericFunctions_orderCircuitList", dsResult, dsCircuit);
}

//------------------------------------------------------------------------------------------------------------------------
/** GenericFunctions_getSynopticName
*/
  
void unLHCCircuit_GenericFunctions_getSynopticName(const string sCircuit, const string sInterlockType, const string sDeviceType, string &sSymbol)
{
string sNewInterlockType;
int iPos;

 if(sDeviceType!="")
  {
/*  //Synoptic with Cryo data - Test!
  if(sCircuit=="RB.A45")
   sSymbol="vision/SYNOPTICS/"+"Synoptic_Interlock_"+sDeviceType+"_S45_CRYO.pnl";
  else if (sCircuit=="RQD.A45")
   sSymbol="vision/SYNOPTICS/"+"Synoptic_Interlock_"+sDeviceType+"_S45_CRYO_D.pnl"; 
  else if (sCircuit=="RQF.A45")
   sSymbol="vision/SYNOPTICS/"+"Synoptic_Interlock_"+sDeviceType+"_S45_CRYO_F.pnl";
  else*/
   sSymbol="vision/SYNOPTICS/"+"Synoptic_Interlock_"+sDeviceType+".pnl";  
  }
 else if(sInterlockType!="")
  {
  if(sInterlockType==CIRCUIT_A)
   {
   if (patternMatch("RQX*", sCircuit))
    sSymbol="vision/SYNOPTICS/"+"Synoptic_Interlock_"+sInterlockType+"_RQX"+".pnl";
   else
    sSymbol="vision/SYNOPTICS/"+"Synoptic_Interlock_"+sInterlockType+".pnl";
   }
  else
   {
   iPos=dynContains(g_dsCircuitException, sCircuit);
   if (iPos>0) sNewInterlockType=g_dsInterlockTypeException[iPos];
   else sNewInterlockType=sInterlockType;
   sSymbol="vision/SYNOPTICS/"+"Synoptic_Interlock_"+sNewInterlockType+".pnl";
   }
  }
 else
  sSymbol="";
}

//------------------------------------------------------------------------------------------------------------------------
/** GenericFunctions_getSynopticDollarParameters
*/
  
void unLHCCircuit_GenericFunctions_getSynopticDollarParameters(const string sInterlockType, const string sCircuitType, const string sCircuit, const dyn_string dsDpNames, const dyn_string dsAliases, const dyn_string dsDpTypes, string &sQPSType, dyn_string &dsQPS, dyn_string &dsPIC, dyn_string &dsPC, dyn_string &dsCBW, string &sCRYO, dyn_string &dsCircuitInfo)
{
int i, iLen, iReduce;
dyn_string dsQPSType, dsQPS_DQAMSRB_Alias, dsQPS_DQAMSRB_DpN, dsSplit, dsCBWTemp, dsPLC, dsTempPC, dsTempPIC, dsTempInstrum, dsTempCryo;
string sLocation, sDQGROUP, sPLC, sSystem, sApplication, sBus;
mapping m;
bool bCPC5;
 
 iLen=dynlen(dsAliases);
 //for each device defined with the circuit (QPS, PIC, PC, CRYO)
 for (i=1; i<=iLen; i++)
  {
  switch(dsDpTypes[i])
   {
   // 0<= x <=5 

   case "DQAMCNMB":
   case "DQAMCNMQ":
   case "DQAMGNSRQF":
   case "DQAMGNSRQD":
   case "DQAMGNSRB":           
   sDQGROUP=unLHCCircuit_QPS_getDQGROUP(dsDpNames[i], sCircuit);
   if(sDQGROUP!="")     
     dynAppend(dsCBWTemp, sDQGROUP);
   break;
 
   //RB.A78.EVEN
   case "DQQLC_NEVEN":    
   sSystem=unGenericDpFunctions_getSystemName(dsDpNames[i]);
   dynAppend(dsQPS, "$QPS_" + dsDpTypes[i] + ":" + sSystem + dsAliases[i]);   
   dynAppend(dsQPSType, dsDpTypes[i]);   
   sLocation=unLHCCircuit_QPS_getDQAMSRBLocation(dsDpNames[i]);
   m[sLocation]="EVEN";   
   break;

   //RB.A78.ODD   
   case "DQQLC_NODD":
   sSystem=unGenericDpFunctions_getSystemName(dsDpNames[i]);
   dynAppend(dsQPS, "$QPS_" + dsDpTypes[i] + ":" + sSystem + dsAliases[i]);
   dynAppend(dsQPSType, dsDpTypes[i]);
   sLocation=unLHCCircuit_QPS_getDQAMSRBLocation(dsDpNames[i]);
   m[sLocation]="ODD";   
   break;
    
   //RR77.ROD.A78B1 or UA83...
   case "DQAMSNRB":
   dynAppend(dsQPS_DQAMSRB_DpN, dsDpNames[i]);
   dynAppend(dsQPS_DQAMSRB_Alias, dsAliases[i]);
   break; 
   
   case "DQAMSNRQ":   
   case "DQAMSN600":    
   //RD2.L8   
   case "DQAMGNC":
   case "DQAMGNDRB_EVEN":
   case "DQAMGNDRB_ODD":
   case "DQAMGNDRQ":   
   case "DQAMGNRQA": 
   case "DQAMGNRQB":
   case "DQAMGNRQC": 
   case "DQAMGNRD":
   case "DQAMGNA":

   sSystem=unGenericDpFunctions_getSystemName(dsDpNames[i]);
   dynAppend(dsQPS, "$QPS_" + dsDpTypes[i] + ":" + sSystem + dsAliases[i]);
   dynAppend(dsQPSType, dsDpTypes[i]);
   break;
    
   case "FGC2_LHC_PC":
   // 1<= x <=3
   sSystem=unGenericDpFunctions_getSystemName(dsDpNames[i]);
   dynAppend(dsTempPC, sSystem+dsAliases[i]);
   
   //Exception: to manage the Interlock Type D for the PIC!!!
   if (sInterlockType==CIRCUIT_D)
    {
    sApplication=unGenericDpFunctions_getApplication(dsDpNames[i]);
    //System Name hardcoded !!!
    dsTempPIC=dpNames(LHC_CIRCUIT_PIC_SYSTEM_NAME + "pic-CFP_*-" + sApplication + "-CIPC-*","CIPC");
    if (dynlen(dsTempPIC)>0)  //How to manage if several CIPC found? Is it possible?
     {
     dyn_string dsCIPAlias;
     string sTitle_PP60A, sCIPC_ALIAS, sSector_PP60A;
     int iSector_PP60A;
     
     sCIPC_ALIAS=unGenericDpFunctions_getAlias(dsTempPIC[1]);
     unLHCCircuit_PIC_getSynopticDollarParameters(sInterlockType, sCircuit, LHC_CIRCUIT_PIC_SYSTEM_NAME, dsTempPIC[1], LHC_CIRCUIT_PIC_SYSTEM_NAME + sCIPC_ALIAS, dsPIC);

     //add info to open the 60A detailed view
     dynAppend(dsPIC, "$sDeviceName:" + LHC_CIRCUIT_PIC_SYSTEM_NAME + CIPC_60A_ALIAS_CALCULATED);     
     
     dsCIPAlias=makeDynString(sCIPC_ALIAS);
     picCIPC_getSectorInfo(dsCIPAlias, iSector_PP60A);
          
     if (iSector_PP60A==11) iSector_PP60A=1;     
     dynAppend(dsPIC, "$iSector:" + iSector_PP60A);

     if(iSector_PP60A==8) sSector_PP60A=(string)iSector_PP60A + "1";
     else sSector_PP60A=(string)iSector_PP60A + (string)(iSector_PP60A+1);      
     dynAppend(dsPIC, "$sTitle_PP60A:" + "PP60A Sector " + sSector_PP60A);     
     }
    }
   break;
   
   case "CIPC":
   // x = 1
   sSystem=unGenericDpFunctions_getSystemName(dsDpNames[i]);
   unLHCCircuit_PIC_getSynopticDollarParameters(sInterlockType, sCircuit, sSystem, dsDpNames[i], sSystem+dsAliases[i], dsPIC);
   break;  

   case "AnalogInput":   
   case "Analog":               
   bCPC5=true;
   sSystem=unGenericDpFunctions_getSystemName(dsDpNames[i]);
   dynAppend(dsTempInstrum, sSystem + dsAliases[i]);
   break;

   case "DigitalInput":               
   bCPC5=true;
   sSystem=unGenericDpFunctions_getSystemName(dsDpNames[i]);
   dynAppend(dsTempCryo, sSystem + dsAliases[i]);
   break;   
        
   case "CPC_AnalogInput":   
   case "CPC_Analog":               
   sSystem=unGenericDpFunctions_getSystemName(dsDpNames[i]);
   dynAppend(dsTempInstrum, sSystem + dsAliases[i]);
   break;
   
   case "CPC_DigitalInput":               
   sSystem=unGenericDpFunctions_getSystemName(dsDpNames[i]);
   dynAppend(dsTempCryo, sSystem + dsAliases[i]);
   break;   
   
   default:
   DebugN("unLHCCircuit_GenericFunctions_getSynopticDollarParameters(): UNKNOWN dpType: " + dsDpTypes[i] + " - Alias: " + dsAliases[i]);
   break;
   }
  }

   //general Circuit information
  dynAppend(dsCircuitInfo, "$CIRCUIT:" + sCircuit); 
  dynAppend(dsCircuitInfo, "$INTERLOCKTYPE:" + sInterlockType); 
  dynAppend(dsCircuitInfo, "$CIRCUITTYPE:" + sCircuitType);  
  
 iLen=dynlen(dsTempPC);
 if (iLen>0)
  unLHCCircuit_PC_getSynopticDollarParameters(dsTempPC, iLen, dsPC);

 iLen=dynlen(dsTempInstrum);
 if (iLen>0)
  unLHCCircuit_CRYO_getInstrumSynopticDollarParameters(sInterlockType, sCircuitType, sCircuit, dsTempInstrum, iLen, sCRYO); 

 iLen=dynlen(dsTempCryo);
 if (iLen>0)
  unLHCCircuit_CRYO_getCryoSynopticDollarParameters(dsTempCryo, iLen, sCRYO);  
  
 //to avoid $parameters errors -> empty $parameters
 if (dynlen(dsPIC)==0)
  unLHCCircuit_PIC_getEmptySynopticDollarParameters(sCircuit, dsPIC);

 if (dynlen(dsQPS)>0)
  {
  dynAppend(dsQPS, "$TITLE_QPS:" + sCircuit);
  if(patternMatch("RB.*", sCircuit)) //special case: to know the side (.ODD or .EVEN) of the 2 DQAMSNRB
   {
   iLen=dynlen(dsQPS_DQAMSRB_DpN);
   for(i=1; i<=iLen; i++)
    {
    sSystem=unGenericDpFunctions_getSystemName(dsQPS_DQAMSRB_DpN[i]);
    sLocation=unLHCCircuit_QPS_getDQAMSRBLocation(dsQPS_DQAMSRB_DpN[i]);
    if(mappingHasKey(m, sLocation))
     {
     dynAppend(dsQPS, "$QPS_DQAMSNRB_" + m[sLocation] + ":" + sSystem + dsQPS_DQAMSRB_Alias[i]);
     dynAppend(dsQPSType, "DQAMSNRB");
     }
    else
     DebugN("unLHCCircuit_GenericFunctions_getSynopticDollarParameters(): Error with the location (not defined in mapping) - Location: " + sLocation + " - Alias: " + dsQPS_DQAMSRB_Alias[i]);
    }
   } 
  
  //Manage the CBW widgets (FIP BUS detail)
  dyn_string dsCBWMix, dsBus, dsRes, dsId;
  string sCBWTemp;
  int j, jLen;
  dynUnique(dsCBWTemp);  
  iLen=dynlen(dsCBWTemp);
  for(i=1; i<=iLen; i++)    //iLen must be equal to 8 if nQPS connected or 4 if nQPS disconnected
    {
    dsCBWMix=strsplit(dsCBWTemp[i],".");
    if(dynlen(dsCBWMix)==4)
      {
      dynAppend(dsBus,dsCBWMix[4]);
      dynAppend(dsId,substr(dsCBWMix[1],strpos(dsCBWMix[1],":")+1));
      } 
    else
      DebugN("unLHCCircuit_GenericFunctions_getSynopticDollarParameters dsCBWTemp - Wrong dsCBWTemp[i]");
    }

  dynUnique(dsBus);
  dynUnique(dsId);
  dynAppend(dsCBW, "$bPopup:TRUE");  
  iLen=dynlen(dsBus);  
  for(i=1; i<=iLen; i++)    //iLen must be equal to 4
    {  
    sCBWTemp="";
    dsRes=dynPatternMatch("*"+dsBus[i], dsCBWTemp);
    jLen=dynlen(dsRes);
    for(j=1;j<=jLen;j++)
      {
      if(sCBWTemp=="") sCBWTemp=dsRes[j];
      else sCBWTemp=sCBWTemp+";"+dsRes[j];
      }
    
    dynAppend(dsCBW, "$QPS_CBW_DQGROUP" + i + ":" + sCBWTemp);
    if(dynlen(dsId)>0)
      {
      string sPanelName="vision/QPS/LHC_QPS_" + dsId[1] + "_ALL_" + dsBus[i] +".pnl";
      dynAppend(dsCBW, "$QPS_CBW_PANEL" + i + ":" + sPanelName);        
      }
    else
      dynAppend(dsCBW, "$QPS_CBW_PANEL" + i + ":");       
    } 
  }
    
 iLen=dynlen(dsQPSType);
 dynSortAsc(dsQPSType);	//need to solve problem on: Ref panel name for Synoptic with ROD.A34.B1, B2 ()not correct order between DQAMSN600 and DQAMGNA
 //According to the QPS devices, define the display
 switch (sInterlockType)
  {
  case CIRCUIT_A:
   if(iLen>0)
    {
    if (patternMatch("RB.*", sCircuit))
     sQPSType=sInterlockType + "_DQAMGNDRB";
    else if (patternMatch("RQX.*", sCircuit))
     sQPSType=sInterlockType + "_DQAMGNC";
    else if (patternMatch("RQ*", sCircuit))
     sQPSType=sInterlockType + "_DQAMGNDRQ";
    }
  break;

  case CIRCUIT_B1:
  case CIRCUIT_B2: 
   for(i=1; i<=iLen; i++)
    {
    if (sQPSType=="")
     sQPSType=sInterlockType+"_"+dsQPSType[i];
    else
     sQPSType=sQPSType+"_"+dsQPSType[i];     
    }     
  break;
  
  case CIRCUIT_C:   
  case "D":
   sQPSType=sInterlockType;     
  break;
    
  default:
   DebugN("unLHCCircuit_GenericFunctions_getSynopticDollarParameters(): UNKNOWN Interlock Type: " + sInterlockType + " - Circuit: " + sCircuit);   
  break;
  }
  // add CPC5/CPC6
  sCRYO+="~$CPC5:"+bCPC5;
  unGenericDpFunctions_debugN(DEBUG_ONLY_BIT0, "unLHCCircuit_GenericFunctions_getSynopticDollarParameters", "unLHCCircuit_GenericFunctions_getSynopticDollarParameters()", sCircuit, sInterlockType, sCircuitType, sCRYO); 
}

//------------------------------------------------------------------------------------------------------------------------
/** display_debug
*/

void display_debug(const string sFilter, const string sText)
{
dyn_string dsFilter;

 dsFilter=strsplit(DEBUG_FILTER, ",");
 if (dynContains(dsFilter, sFilter)>0)
  DebugTN(sText);
  
}

//------------------------------------------------------------------------------------------------------------------------
/** GenericFunctions_widgetCB_checkParameters
*/

bool unLHCCircuit_GenericFunctions_widgetCB_checkParameters(const string sText, const int iNum)
{ 
bool bOk;
string sInterlockType, sCircuitType;
int istrlen, iPos;
 
 istrlen=strlen(sText);

  // Powering SubSector / For each Pw Sb: nQPS-QPS-PIC and PC PVSS systems
 	if ((istrlen==3) && iNum==mappinglen(g_mDeviceIdentifier))
		bOk=TRUE;
  // Powering SubSector / For each Pw Sb: QPS-nQPS-PIC and PC PVSS systems
// 	else if ((istrlen==3) && iNum==LHC_CIRCUIT_PVSS_SYSTEMS_NQPS)
//		bOk=TRUE;
  // Circuit Name                
	else if (istrlen>3)
		{	
		iPos=dynContains(g_dsCircuitException, sText);
		if (iPos>0) sInterlockType=g_dsInterlockTypeException[iPos];
    else
      {
      DebugN("unLHCCircuit_GenericFunctions_widgetCB_checkParameters - ? used?", sText);
      unLHCCircuit_GenericFunctions_getCircuitType(sText, sInterlockType, sCircuitType);
    	}
      
		if(sInterlockType!="")
			{
			switch (sInterlockType)
				{
				case CIRCUIT_A:
				case CIRCUIT_B1:
				case CIRCUIT_B2:
				bOk=(iNum==3);
				break;
					
				case CIRCUIT_C:
				bOk=(iNum==2);
				break;
				
				case CIRCUIT_D:			
				bOk=(iNum==1);			
				break;
				
				default:
				bOk=FALSE;
				DebugN("unLHCCircuit_GenericFunctions_widgetCB_checkParameters()! Unknown Interlock Type: " + sInterlockType);
				}
			}
		else
			{
			DebugN("unLHCCircuit_GenericFunctions_widgetCB_checkParameters()! No Interlock Type defined");
			bOk=FALSE;
			}
		}
	// CBW standard 
	else if ((istrlen==0) && iNum==1)
		bOk=TRUE;
	// CBW standard  with nQPS
	else if ((istrlen==0) && iNum==2)
		bOk=TRUE;        
	else	
		bOk=FALSE;
	
return bOk;
}
//------------------------------------------------------------------------------------------------------------------------
/** GenericFunctions_checkParametersPerInterlockType
*/

bool unLHCCircuit_GenericFunctions_checkParametersPerInterlockType(const string sCircuit, const string sInterlockType, const int iLenQPS, const int iLenPIC, const int iLenPC)
{	
bool bOk;

	switch (sInterlockType)
		{
		case CIRCUIT_A:
			if (patternMatch("RQX*", sCircuit))
				bOk=(iLenQPS==1 && iLenPIC==1 && iLenPC==3);
			else
				bOk=(iLenQPS==6 && iLenPIC==1 && iLenPC==1);
			break;
		
		case CIRCUIT_B1:
			bOk=(iLenQPS==1 && iLenPIC==1 && iLenPC==1);
			break;
		
		case CIRCUIT_B2:
			bOk=(iLenQPS==1 && iLenPIC==1 && iLenPC==2);
			break;
		
		case CIRCUIT_C:
			bOk=(iLenQPS==0 && iLenPIC==1 && iLenPC==1);
			break;

		case CIRCUIT_D:
			bOk=(iLenQPS==0 && iLenPIC==0 && iLenPC==1);
			break;
		
		default:
			bOk=FALSE;
			DebugN("unLHCCircuit_GenericFunctions_checkParametersPerInterlockType(): unknown Interlock Type " + sInterlockType);
		}

return bOk;
}

//-----------------------------------------------------------------------------------------
/** GenericFunctions_sizeCircuitWidget
*/

void unLHCCircuit_GenericFunctions_sizeCircuitWidget()
{
dyn_int diDim, diAliasDim;
		
	diDim = getPanelSize("objects/LHC_CIRCUIT_DEVICES/unCIRCUITWidget.pnl");
	if (dynlen(diDim) >= 2)
		{
		g_width = diDim[1];
		g_height = diDim[2];
		}
	else
		{
		g_width = LHC_CIRCUIT_DEFAULTWIDTH_CIRCUIT;
		g_height = LHC_CIRCUIT_DEFAULTHEIGHT_CIRCUIT;
		}

	g_columns = g_iCIRCUIT_BASE_PANEL_WIDTH / g_width;
	g_lines = (g_iCIRCUIT_BASE_PANEL_HEIGHT - CIRCUIT_TOP_SPACING) / g_height;
			
	g_leftSpacing = (g_iCIRCUIT_BASE_PANEL_WIDTH - (g_columns * g_width)) / 2;
	g_topSpacing = ((g_iCIRCUIT_BASE_PANEL_HEIGHT - CIRCUIT_TOP_SPACING) - (g_lines * g_height)) / 2;
}

//-----------------------------------------------------------------------------------------
/** GenericFunctions_distributedRegister
*/

void unLHCCircuit_GenericFunctions_distributedRegister(const string sSystem, const string sIdentifier)
{	
string sDpPanel, sPanelName;
bool bConnected, bInitOk = true;
dyn_string exceptionInfoTemp, exceptionInfo;
string sTreeDevOvDp=unTreeDeviceOverview_getInternalTriggerName();
	
	if(dpExists(sTreeDevOvDp))
		{
		if (strltrim(sIdentifier) != "")
			unTreeDeviceOverview_distributedRegister("unLHCCircuit_GenericFunctions_TreeDeviceOverviewWidgetRegisterCB", bInitOk, bConnected, sSystem, sTreeDevOvDp, exceptionInfo);
		else
			bInitOk = false;
		}
	else
			bInitOk = false;

	if (!bInitOk)
		{
		unGraphicalFrame_getPanel(myModuleName(), sDpPanel, sPanelName);
		unSendMessage_toAllUsers(sIdentifier + getCatStr("unOperator","UNKNOWNDOLLARPARAM") + " in panel " + sPanelName, exceptionInfoTemp);
		if (dynlen(exceptionInfoTemp) > 0)
			fwExceptionHandling_display(exceptionInfoTemp);

		fwException_raise(exceptionInfo, "EXPERTINFO", "$sIdentifier" + getCatStr("unPVSS","UNKNOWNDOLLARPARAM") + " or register failed", "");
		unSendMessage_toExpertException("unGenericObject_WidgetInit", exceptionInfo);
		}	
}

//-----------------------------------------------------------------------------------------
/** GenericFunctions_displayCircuits
*/

void unLHCCircuit_GenericFunctions_displayCircuits(const dyn_string dsWidgetFileNameList, const dyn_string dsInterlockType, const dyn_string dsCircuitType)
{
int positionX, positionY;
int iLeft, iTop, i, iLen, iMax;
	
	loadingText.visible=TRUE;
	
	panelSize("", g_iCIRCUIT_BASE_PANEL_WIDTH, g_iCIRCUIT_BASE_PANEL_HEIGHT);		
	unLHCCircuit_GenericFunctions_sizeCircuitWidget();

	iMax=LHC_CIRCUIT_WIDGET_PANEL_NB_CIRCUIT_MAX;//not possible to do it in the function call 
	iLen=dynlen(dsWidgetFileNameList);
		
	if (iLen>0)
		{		
		if (iLen>iMax) iLen=iMax;

		positionX = 1;
		positionY = 1;
		
		for(i=1; i<=iLen; i++)
			{
			iLeft = g_leftSpacing + (g_width * (positionX - 1));
			iTop = CIRCUIT_TOP_SPACING + g_topSpacing + (g_height * (positionY - 1));
		
			unLHCCircuit_GenericFunctions_addCircuit(i, dsWidgetFileNameList[i], dsInterlockType[i], dsCircuitType[i], iLeft, iTop);						
			
			positionX++;
			if (positionX > g_columns)
				{
				positionX = 1;
				positionY++;
				}
			}
		}
	else
		DebugN("unLHCCircuit_GenericFunctions_displayCircuits(): NO CIRCUT DEFINED!");

	loadingText.visible=FALSE;

}

//-----------------------------------------------------------------------------------------
/** GenericFunctions_displaySynoptics
*/

void unLHCCircuit_GenericFunctions_displaySynoptics(const string sCircuit, const string sInterlockType, const string sCircuitType, const int x, const int y, const int h)
{
string sSymbol, sQPSType, sModule, sPanel, sCryo;
dyn_string dsPIC, dsPC, dsQPS, dsDollars, dsCircuits, dsInterlockType, dsCircuitType, dsCBW, dsCircuitInfo;
dyn_dyn_string ddsDeviceType, ddsDp, ddsAlias;
int iPos, yPos, iLen, i;
			
		dsCircuits=strsplit(sCircuit, ",");				
		dsInterlockType=strsplit(sInterlockType, ",");
		dsCircuitType=strsplit(sCircuitType, ",");
    iLen=dynlen(dsCircuits);		
		if(iLen>0)
			{						
			sModule = myModuleName();
	    sPanel = myPanelName(); 

			if(dynlen(dsInterlockType)!=iLen)
				{
				dynClear(dsInterlockType);
				unLHCCircuit_GenericFunctions_getCircuitType(dsCircuits, dsInterlockType, dsCircuitType);
				}

			//0bis. Get all the devices for all the circuits
			unLHCCircuit_GenericFunctions_getCircuitDevice (dsCircuits, ddsDp, ddsAlias, ddsDeviceType);

			if (iLen==dynlen(ddsDp))
				{
				//for each circuit
				for(i=1; i<=iLen; i++)
					{						
					//1. Init
					dynClear(dsDollars);
					dynClear(dsPIC);				
					dynClear(dsPC);
					dynClear(dsQPS);
          dynClear(dsCBW);                                        
          sCryo="";
                                                  				
					yPos=y+((i-1)*h);
					iPos=x;
					sQPSType="";
					sSymbol="";
					
					if(dsInterlockType[i]!="")
						{	
						unLHCCircuit_GenericFunctions_getSynopticDollarParameters(dsInterlockType[i], dsCircuitType[i], dsCircuits[i], ddsDp[i], ddsAlias[i], ddsDeviceType[i], sQPSType, dsQPS, dsPIC, dsPC, dsCBW, sCryo, dsCircuitInfo);
            unGenericDpFunctions_debugN(DEBUG_ONLY_BIT0, dsCircuits[i]+" getSynopticDollar", "getSynopticDollar "+myPanelName() + ": " + sSymbol, sQPSType, dsQPS, dsPIC, dsPC, dsCBW, sCryo, dsCircuitInfo, ddsDp[i], ddsAlias[i], ddsDeviceType[i], "-end-");
						unLHCCircuit_GenericFunctions_checkAndCorrectDollarParam(dsInterlockType[i], dsCircuitType[i], dsCircuits[i], dsCBW);
						//3. Add the Synoptic symbol
						dynAppend(dsDollars, dsPIC);
						dynAppend(dsDollars, dsPC);		
						dynAppend(dsDollars, dsQPS);		
            dynAppend(dsDollars, dsCBW);			
            dynAppend(dsDollars, "$CRYO_DOLLARS:" + sCryo);
            dynAppend(dsDollars, dsCircuitInfo);            
                                                                        
						//4. Define the Symbol(panel) according to the circuit type
						unLHCCircuit_GenericFunctions_getSynopticName(dsCircuits[i], dsInterlockType[i], sQPSType, sSymbol);
												
						if (sSymbol!="")
						  {	
						  dynAppend(dsDollars, unLHCCircuit_GenericFunctions_getHTMLLink_DollarParam());
              //DebugTN(sPanel, dsCircuits[i], dsDollars);                                                                
						  addSymbol(sModule, sPanel, sSymbol, dsCircuits[i], dsDollars, iPos, yPos, 0, 1, 1);
						  }
		        else
		          setMultiValue("txtMessage", "visible", TRUE, "txtMessage", "text", "NO PANEL DEFINED FOR THE CIRCUIT TYPE: " + dsInterlockType[i]);	
		        }
					else
					  setMultiValue("txtMessage", "visible", TRUE, "txtMessage", "text", "CIRCUIT (" + dsCircuits[i] + ") " + "TYPE UNKNOWN");
					}
				}
			else
				txtMessage.visible=TRUE;
      }                        
 }

//------------------------------------------------------------------------------------------------------------------------
// unLHCCircuit_GenericFunctions_checkAndCorrectDollarParam
/** correct the $-param for the QPS DQROUP for RB, RQF and RQD circuit: QPS and nQPS DQGROUP are needed
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui

@param sInterlockType  input, the circuit interlock type
@param sCircuitType  input, the circuit type
@param sCircuit  input, the circuit
@param dsCBW  output, the $-param
*/
unLHCCircuit_GenericFunctions_checkAndCorrectDollarParam(string sInterlockType, string sCircuitType, string sCircuit, dyn_string &dsCBW)
{
  dyn_string dsTemp = dsCBW;
  int i, len;
  bool bReset;
  string sTemp;
  int iPos_QPS, iPos_Dollar;
  
  switch(sInterlockType)
  {
    case CIRCUIT_A:
      if((strpos(sCircuit, "RB.A") == 0) || (strpos(sCircuit, "RQD.A") == 0) || (strpos(sCircuit, "RQF.A") == 0))
      {
        len = dynlen(dsTemp);
        for(i=1;i<=len;i++)
        {
          iPos_Dollar = strpos(dsTemp[i], "$QPS_CBW_DQGROUP");
          if(iPos_Dollar>=0)
          {
            sTemp = substr(dsTemp[i], strlen("$QPS_CBW_DQGROUP")+2, strlen(dsTemp[i])-(strlen("$QPS_CBW_DQGROUP")+2));
            iPos_QPS = strpos(sTemp, "QPS_");
            if(iPos_QPS <0)
            {
              bReset = true;
              strreplace(dsTemp[i], sTemp, "no-device");
            }
          }
        }
        if(bReset)
        {
          unGenericDpFunctions_debugN(DEBUG_ONLY_BIT0, sCircuit+" checkAndCorrectDollarParam", "checkAndCorrectDollarParam: missing DQGROUP", dsCBW, dsTemp, "-end-");
          dsCBW = dsTemp;
        }
      }
      break;
    default:
      break;
  }
}

//-----------------------------------------------------------------------------------------
/** GenericFunctions_addCircuit
*/

void unLHCCircuit_GenericFunctions_addCircuit(const int iIndex, const string sWidgetFileNameList, const string sInterlockType, const string sCircuitType, const int iLeft, const int iTop)
{
  unGenericDpFunctions_debugN(DEBUG_ONLY_BIT0, "unLHCCircuit_GenericFunctions_addCircuit", iIndex, iLeft, iTop, "Adding circuit "+ sWidgetFileNameList+","+sInterlockType+" in "+myModuleName()+" "+myPanelName());
  addSymbol(myModuleName(), myPanelName(), "objects/LHC_CIRCUIT_DEVICES/unCIRCUITWidget.pnl", WIDGET_REFERENCE + iIndex, makeDynString("$sCircuit:" + sWidgetFileNameList, "$sInterlockType:" + sInterlockType, "$sCircuitType:" + sCircuitType, "$sTitle:Circuit Synoptic: " + sWidgetFileNameList, "$bPopup:TRUE"), iLeft, iTop, 0, 1, 1);	
}

//-----------------------------------------------------------------------------------------
/** GenericFunctions_getDpNParameters
*/

bool unLHCCircuit_GenericFunctions_getDpNParameters(const int iCircuit, const string sCircuit, const string sInterlockType, dyn_string &dsDollars)
{
int iLen, i, iLenQPS, iLenPIC, iLenPC;
string sTemp, sLink;
dyn_string dsDQGroup;

	dynClear(dsDollars);
	iLenQPS=0;
	iLenPIC=0;
	iLenPC=0;
	iLen=dynlen(g_ddsDpN);
		
	if (iCircuit<=iLen)
		{
		iLen=dynlen(g_ddsDpN[iCircuit]);		
		for (i=1; i<=iLen; i++)
			{
			switch (g_ddsType[iCircuit][i])
				{
				case "FGC2_LHC_PC":
					dynAppend(dsDollars, g_ddsDpN[iCircuit][i]);
					iLenPC++;
				break;
				
				case "CIPC":
					dynAppend(dsDollars, g_ddsDpN[iCircuit][i]);
					iLenPIC++;
				break;
					
				case "DQAMCNMB":
				case "DQAMCNMQ":
				case "DQQLC_NEVEN":				
				case "DQQLC_NODD":
				case "DQAMSNRB":
				case "DQAMSNRQ":			
				case "DQAMSN600":				
				case "DQAMGNC":
				case "DQAMGNDRB_EVEN":
				case "DQAMGNDRB_ODD":
				case "DQAMGNDRQ":			
				case "DQAMGNRQA":	
				case "DQAMGNRQB":
				case "DQAMGNRQC":	
				case "DQAMGNRD":
				case "DQAMGNA":					
					unGenericDpFunctions_getDeviceLink(g_ddsDpN[iCircuit][i], sLink);
					if (sLink!="")
						{
						if (!dynContains(dsDQGroup, sLink))
							{
							sTemp=unGenericDpFunctions_getWidgetDpName(unGenericDpFunctions_getSystemName(g_ddsDpN[iCircuit][i]) + sLink);
							if (sTemp!="")
								{
								dynAppend(dsDollars, sTemp);
								dynAppend(dsDQGroup, sLink);
								iLenQPS++;
								}
							}
						}
				break;
				}			
			}
		}
		
	unGenericDpFunctions_debugN(DEBUG_ONLY_BIT0, sCircuit, "_GenericFunctions_getDpNParameters", dsDollars);
	
	return !unLHCCircuit_GenericFunctions_checkParametersPerInterlockType(sCircuit, sInterlockType, iLenQPS, iLenPIC, iLenPC);	
}

//-----------------------------------------------------------------------------------------
/** GenericFunctions_TreeDeviceOverviewWidgetRegisterCB
*/

void unLHCCircuit_GenericFunctions_TreeDeviceOverviewWidgetRegisterCB(string sDp, bool bConnected, 
																																			string sTreeDp, int iCommand,
																																			string sTreeDpSystem, string sSystemName)
{
dyn_string dsDollar;
mapping mCircuitIdentifier_old;

	//DebugN("unLHCCircuit_GenericFunctions_TreeDeviceOverviewWidgetRegisterCB", bConnected, iCommand, sSystemName);
	
		if(iCommand==1 || g_bInit)
			{	
			mCircuitIdentifier_old=g_mCircuitIdentifier;
			unLHCCircuit_GenericFunctions_WidgetRegister();			
			if (!unGenericObject_compareMapping(mCircuitIdentifier_old, g_mCircuitIdentifier) || g_bInit)
				{				
				dsDollar = unLHCCircuit_buildDollarModule();
				dpSet("_Ui_"+myManNum()+".RootPanelOrigOn.ModuleName", myModuleName()+g_sPanelName,
							"_Ui_"+myManNum()+".RootPanelOrigOn.FileName", g_sFileName,
							"_Ui_"+myManNum()+".RootPanelOrigOn.PanelName", g_sPanelName,
							"_Ui_"+myManNum()+".RootPanelOrigOn.Parameter", dsDollar);
				}
			if(g_bInit) g_bInit=FALSE;				
			}
}

//-----------------------------------------------------------------------------------------
/** buildDollarModule
*/

dyn_string unLHCCircuit_buildDollarModule()
{
int i, iLen;
string sCircuits, sInterlockTypes;
dyn_string dsDollars;

	iLen=dynlen(g_dsCircuitsList);
	if(iLen==dynlen(g_dsInterlockType))
		{
		for(i=1; i<=iLen; i++)
			{
			sCircuits=sCircuits+g_dsCircuitsList[i]+",";
			sInterlockTypes=sInterlockTypes+g_dsInterlockType[i]+",";
			}
		}
		
	dsDollars=makeDynString("$sCircuits:"+sCircuits,"$sInterlockTypes:"+sInterlockTypes);
	return dsDollars;
}

//-----------------------------------------------------------------------------------------
/** getDollarModule
*/

void unLHCCircuit_getDollarModule(const string sCircuits, const string sInterlockTypes, const string sCircuitTypes, dyn_string &dsCircuits, dyn_string &dsInterlockTypes, dyn_string &dsCircuitTypes)
{
	dsCircuits=strsplit(sCircuits, ",");
	dsInterlockTypes=strsplit(sInterlockTypes, ",");
  dsCircuitTypes=strsplit(sCircuitTypes, ",");  
        
	if(dynlen(dsCircuits)!=dynlen(dsInterlockTypes))
		{
		dsCircuits=makeDynString();
		dsInterlockTypes=makeDynString();
    dsCircuitTypes=makeDynString();                
		}
}

//-----------------------------------------------------------------------------------------
/** GenericFunctions_getSystemNameInvolved
*/

dyn_string unLHCCircuit_GenericFunctions_getSystemNameInvolved()
{
int i, iLen, j, jLen;
string sSystem;
dyn_string dsDevices, dsSystems;

	iLen=mappinglen(g_mCircuitIdentifier);
	for (i=1; i<=iLen; i++)	//for each Circuit
		{
		dsDevices=mappingGetValue(g_mCircuitIdentifier,i);
		jLen=dynlen(dsDevices);
		for (j=1; j<=jLen; j++)	//for each Device
			{
			sSystem=unGenericDpFunctions_getSystemName(dsDevices[j]);
			if(dynContains(dsSystems, sSystem)==0)
				dynAppend(dsSystems, sSystem);
			}
		}

	return dsSystems;
}

//-----------------------------------------------------------------------------------------
/** GenericFunctions_WidgetRegister
*/

void unLHCCircuit_GenericFunctions_WidgetRegister()
{
dyn_string dsDeviceName;
int jLen, j;
bool bError;
	
	unLHCCircuit_GenericFunctions_getCircuitDevice(g_dsCircuitsList, g_ddsDpN, g_ddsAlias, g_ddsType);
	
	jLen=dynlen(g_dsCircuitsList);
	for(j=1; j<=jLen; j++)
		{
		bError=unLHCCircuit_GenericFunctions_getDpNParameters(j, g_dsCircuitsList[j], g_dsInterlockType[j], dsDeviceName);
		g_mCircuitIdentifier[g_dsCircuitsList[j]]=dsDeviceName;	
		}

	//g_dsSystemInvolved=unLHCCircuit_GenericFunctions_getSystemNameInvolved();

	//DebugN("unLHCCircuit_GenericFunctions_WidgetRegister", jLen);			
}

//-----------------------------------------------------------------------------------------
/** GenericFunctions_getHTMLLink_DollarParam
*/

string	unLHCCircuit_GenericFunctions_getHTMLLink_DollarParam()
{
string sPath, sChar, sDollar;
dyn_string dsPath;

	sDollar="$HTML_HELP:";
	sPath=getPath(HELP_REL_PATH);	
	dsPath=strsplit(sPath, "/");
	if(dynlen(dsPath)==7)
		{
    if(_UNIX)                    
      {
  		sPath= "/nfs/" + dsPath[3] + "/local/unicryo/" +dsPath[4] + "/" + dsPath[5] + "/installed_components/" + dsPath[7] + "/" + HTML_HELP;
  		sDollar=sDollar+sPath;      
      }
    else
      {            
  		sPath= "//" + dsPath[3] + "/" +dsPath[4] + "/" + dsPath[5] + "/installed_components/" + dsPath[7] + "/" + HTML_HELP;
  		sChar= "\\";
  		strreplace(sPath, "/", sChar);
  		sDollar=sDollar+sPath;
      }                
		}
	
	//DebugN("unLHCCircuit_GenericFunctions_getHTMLLink_DollarParam(): " + sDollar);
	return sDollar;
}

//-----------------------------------------------------------------------------------------                                
// unLHCCircuit_GenericFunctions_CircuitModule
/**
Purpose: special Behavior for LHC_CIRCUIT when opening a Module (from Circuit Widget)

Parameters:
  - dsDollarParameters: input, dyn_string, list of $-param
  - sPanelFileName: input, string, name of the reference panel
  - sTitle: input, string, name of circuit
*/
void unLHCCircuit_GenericFunctions_CircuitModule(string sPanelFileName, const string sTitle, const dyn_string dsDollarParameters)
{    
string sModule;
bool bModuleOk;
string sInterlockType, sCircuitType;
dyn_string dsPanelFileName, dsSplit;

                              sModule="Circuit_" + sTitle;
                              strreplace(sModule, ".", "_");
                              bModuleOk=!isModuleOpen(sModule);
                              if(g_iOpenedModule>=g_iMaxOpenedModule && bModuleOk)
                                {
                                ChildPanelOnCentralModal("vision/MessageWarning", "Warning", makeDynString("$1:The limit of " + g_iMaxOpenedModule + " opened windows is reached. Please close any if you want to open a new one."));
                                bModuleOk=FALSE;
                                }
                               
                              if(bModuleOk && dynlen(dsDollarParameters)==4)
                                {         
                                sInterlockType=dsDollarParameters[4];
                                dsSplit=strsplit(sInterlockType,":");
                                sInterlockType=dsSplit[2];                                
                                sCircuitType=dsDollarParameters[3];
                                dsSplit=strsplit(sCircuitType,":");
                                sCircuitType=dsSplit[2];                                
                                dsPanelFileName=strsplit(sPanelFileName, ".");
                                if (dynlen(dsPanelFileName)==2)
                                  {
                                  sPanelFileName=dsPanelFileName[1]+"_"+sInterlockType+"."+dsPanelFileName[2];
                                  ModuleOnWithPanel(sModule, 100, 200, 0, 0, 1, 1, "", sPanelFileName, " ", dsDollarParameters);
				                          stayOnTop(true, sModule);
                                  g_iOpenedModule++;
                                  }
                                else
                                  DebugN("WARNING - unLHCCircuit_GenericFunctions_CircuitModule() - Incorrect panel name! " + sPanelFileName);                                                                                                                                                                                                                                                                        
                                }
}

//------------------------------------------------------------------------------------------------------------------------
// unLHCCircuit_GenericFunctions_PWScloseSwitch
/**
Purpose: Close all the switches of a powering sub-sectors, function called on a RC from the PWS widget

Parameters:
  - dsDollarParameters: input, dyn_string, list of $-param given to the PWS widget
  - sRef: input, string, reference given to the PWS widget
*/
void unLHCCircuit_GenericFunctions_PWScloseSwitch(dyn_string dsDollarParameters, string sRef)
{
  string sPWS;
  dyn_string dsMenuList;
  int iAnswer;
  string sGranted="0";
  dyn_string exceptionInfo;
  bool bGranted;
  
//DebugN("unLHCCircuit_GenericFunctions_PWScloseSwitch", dsDollarParameters, sRef);
  if(dynlen(dsDollarParameters) > 0) {
    sPWS=substr(dsDollarParameters[1], strpos(dsDollarParameters[1],":") +1, strlen(dsDollarParameters[1]));
    if(sPWS != "") {
      bGranted = qpsGenericFunctions_PWScloseSwitch_Allowed();
      if(bGranted)
        sGranted="1";
      dynAppend(dsMenuList, "PUSH_BUTTON, Close all the switches in "+ sPWS+", 1, "+sGranted);
	
      popupMenu(dsMenuList, iAnswer);
      switch (iAnswer)
      {
        case 1:
          qpsGenericFunctions_closeSwitchMacro("*", sPWS);
          break;	
        default:
          break;
      }
//DebugN("unLHCCircuit_GenericFunctions_PWScloseSwitch", sPWS);
    }
  }
}

//------------------------------------------------------------------------------------------------------------------------
/** GenericFunctions_setDefaultUser
*/

int unLHCCircuit_GenericFunctions_setDefaultUser()
{
    DebugTN("unLHCCircuit_GenericFunctions_setDefaultUser: no domain, no group and no user");
	return 0;
}

//------------------------------------------------------------------------------------------------------------------------
/** GenericFunctions_setFileAccess
*/

int unLHCCircuit_GenericFunctions_setFileAccess()
{
dyn_string dsFileAccess, dsMenu;
int iRes;
  
  unLHC_Circuit_getDefaultFileAccessControl(dsFileAccess);

  unLHC_Circuit_getDefaultMenuConfiguration(dsMenu);
  
  iRes=dpSetWait(	"_unApplication.menu.menuFileAccessControl", dsFileAccess,
					"_unApplication.menu.menuConfiguration", dsMenu);
  
  if (iRes>=0)
    DebugTN("unLHCCircuit_GenericFunctions_setFileAccess: File Access and Menu are set up for the PIC application");
  else
    DebugTN("unLHCCircuit_GenericFunctions_setFileAccess: File Access and Menu set up failed");

  return iRes; 
}

//------------------------------------------------------------------------------------------------------------------------
// unLHC_Circuit_getDefaultMenuConfiguration
/** returns the menu configuration
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param dsMenu output, return data, the menu definition
*/
unLHC_Circuit_getDefaultMenuConfiguration(dyn_string &dsMenu)
{
  dsMenu=makeDynString("Configuration;text;Configuration   ;",
            "Configuration;insertItemId;;0;Application (local);vision/graphicalFrame/applicationConfiguration.pnl;;AccessAdmin;",
            "Configuration;insertItemId;;0;Tree Management (local);vision/graphicalFrame/unicosSystemDPs.pnl;;AccessExpert;",
            "Configuration;insertItemId;;0;Window Tree (local);vision/windowTree/unWindowTreeConfiguration.pnl;;AccessAdmin;",
            "Configuration;insertItemId;;0;Trend Tree (local);vision/trendTree/unTrendTreeConfiguration.pnl;;AccessExpert;",
            "Configuration;insertItemId;;0;System integrity (local);vision/systemIntegrity/systemIntegrityConfiguration.pnl;;AccessAdmin;",
            "Configuration;insertItemId;;0;Mail/SMS (local);vision/systemIntegrity/unConfigMail.pnl;;AccessAdmin;",
            "Configuration;insertItemId;;0;Backup (local);vision/systemIntegrity/unBackup.pnl;;AccessAdmin;",
            "Configuration;insertItemId;;0;Devices (local);vision/unicosObject/General/unModifyConfig.pnl;;AccessAdmin;",
            "Configuration;insertItemId;;0;Device DPEs(local);vision/unicosObject/General/unSetDeviceDPEConfigs.pnl;;AccessAdmin;",
            "Configuration;insertItemId;;0;Device Types (local);vision/unicosObject/General/unTypeConfigurator.pnl;;AccessAdmin;",
            "Configuration;insertItemId;;0;LHC Logging;vision/unLHCLogging/unLHCLoggingManager.pnl;unLHCLogging;AccessAdmin;",
            "Configuration;insertItemId;;0;LHC Laser;vision/unLaser/unLaserManager.pnl;unLHCLASER;AccessAdmin;",
            "Configuration;insertItemId;;0;CMW Client;vision/unCMWClient/unCMWClientDriver.pnl;unCMWClient;AccessAdmin;",
            "Configuration;insertItemId;;0;CMW Server;vision/unCMWServer/unCMWServerManager.pnl;unCMWServer;AccessAdmin;",
            "Configuration;insertItemId;;0;Distributed System (local);vision/distributedControl/unDistributedControl_configuration.pnl;;AccessAdmin;",
            "Configuration;insertItemId;;0;File access control (local);vision/graphicalFrame/unFileAccessControlConfiguration.pnl;;AccessAdmin;",
            "Configuration;insertItemId;;0;Menu configuration (local);vision/graphicalFrame/unMenuConfiguration.pnl;;AccessAdmin;",
            "Management;text;Management  ;",
            "Management;insertItemId;;1;Utilities (local);;;;",
            "Management;insertItemId;Utilities (local);0;Import Database (local);vision/unGeneration/unGenerateDatabase.pnl;;AccessAdmin;",
            "Management;insertItemId;Utilities (local);0;Import Tree (local);vision/unGeneration/unImportTree.pnl;;AccessAdmin;",
            "Management;insertItemId;Utilities (local);0;Value archive;para/archive/archive_select.pnl;;AccessAdmin;",
            "Management;insertItemId;Utilities (local);0;System management;vision/SysMgm;;AccessExpert;",
            "Management;insertItemId;Utilities (local);0;Reset managers;vision/systemIntegrity/stopDriverAndCtrl.pnl;;AccessAdmin;",
            "Management;insertItemId;Utilities (local);0;Export Database;vision/unGeneration/unExportDatabase.pnl;;AccessOperator;",
            "Management;insertItemId;Utilities (local);0;Export Tree;vision/unGeneration/unExportTree.pnl;;AccessOperator;",
            "Management;insertItemId;Utilities (local);0;Debug Utility;vision/unCore/unDebugUtility.pnl;;AccessAdmin;",
            "Management;insertItemId;;1;Diagnostic;;;;",
            "Management;insertItemId;Diagnostic;0;System Overview (local);vision/Redundanz;;;",
            "Management;insertItemId;Diagnostic;0;Disk space (local);vision/Device.pnl;;;",
            "Management;insertItemId;Diagnostic;0;Memory (local);vision/DeviceVirtual.pnl;;;",
            "Management;insertItemId;Diagnostic;0;Ev connections (local);vision/Connection.pnl;;;",
            "Management;insertItemId;Diagnostic;0;Log statistics;vision/UserConnections;;;",
            "Management;insertItemId;Diagnostic;0;Distributed System (local);vision/distributedControl/unDistributedControl_operation.pnl;;;",
            "Management;insertItemId;Diagnostic;0;Application;vision/graphicalFrame/applicationOperation.pnl;;;",
            "Management;insertItemId;Diagnostic;0;Front-End;vision/graphicalFrame/plcDiagnostic.pnl;;;",
            "Management;insertItemId;Diagnostic;0;UNICOS select;vision/select/unSelectDeselect_diagnostic.pnl;;;",
            "Management;insertItemId;Diagnostic;0;System integrity;vision/systemIntegrity/systemIntegrityOperation.pnl;;;",
            "Management;insertItemId;Diagnostic;0;System ALarm statistics;vision/systemIntegrity/unSystemIntegrity_getStats.pnl;;;",
            "Management;insertItemId;Diagnostic;0;LHC Logging;vision/unLHCLogging/unLHCLoggingStatus.pnl;unLHCLogging;AccessOperator;",
            "Management;insertItemId;Diagnostic;0;LHC Laser;vision/unLaser/unLaserStatus.pnl;unLHCLASER;AccessOperator;",
            "Management;insertItemId;Diagnostic;0;CMW Client;vision/unCMWClient/unCMWClientStatus.pnl;unCMWClient;AccessOperator;",
            "Management;insertItemId;Diagnostic;0;CMW Server;vision/unCMWServer/unCMWServerStatus.pnl;unCMWServer;AccessOperator;",
            "Management;insertItemId;Diagnostic;0;Application Overview (local);vision/graphicalFrame/unApplicationConfigurationOverview.pnl;;;",
            "Management;insertItemId;Diagnostic;0;Archives;vision/unDiagnostics/unArchiveStatistics.pnl;;AccessAdmin;",
            "Management;insertItemId;Diagnostic;0;User Interfaces;vision/unDiagnostics/unUIOverview.pnl;;AccessOperator;",
            "Management;insertItemId;;0;List of components (local);fwInstallation/fwInstallationList.pnl;;;");  
}

//------------------------------------------------------------------------------------------------------------------------
// unLHC_Circuit_getDefaultFileAccessControl
/** returns the file access control configuration
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param dsFileAccess output, return data, the file access control configuration
*/
unLHC_Circuit_getDefaultFileAccessControl(dyn_string &dsFileAccess)
{
  dsFileAccess=makeDynString(
                  "vision/picDevices/CIPC/picCIPC_Access.pnl|INTERLOCK||||",
                  "vision/picDevices/CIPC/picCIPC_PP60A.pnl|INTERLOCK||||",  
                  "vision/systemIntegrity/vision/unicosObject/General/unSetDeviceDPEConfigs.pnl|UNICOS|||AccessAdmin|",  
                  "vision/systemIntegrity/unBackup.pnl|UNICOS|||AccessAdmin|",      
                  "vision/unCore/unDebugUtility.pnl|UNICOS|||AccessAdmin|",
                  "vision/qpsDevices/common/qpsCloseSwitch.pnl|LHCOP,QPS||AccessExpert||",
                  "vision/picDevices/CIPC/picCIPC_UPSOK.pnl|INTERLOCK||||",
                  "vision/picCIPC_GlobalCmd.pnl|INTERLOCK||||",
                  "vision/unLHC_Interlock_QPS_EventList/unLHC_Interlock_QPS_EventList.pnl|INTERLOCK|||AccessAdmin|",  
                  "vision/picCIPC_PMModeSelect.pnl|INTERLOCK|||AccessAdmin|",
                  "vision/picDevices/CIPC/picCIPC_force.pnl|INTERLOCK|||AccessAdmin|",
                  "vision/picDevices/CIPC/picFaceplateCIPC_Monitoring.pnl|INTERLOCK|||AccessAdmin|",
                  "vision/picDevices/CIPC/picFaceplateCIPC_Powering.pnl|INTERLOCK||||",
                  "vision/picDevices/CIPC/picFaceplateCIPC_Note.pnl|INTERLOCK|AccessOperator|||",
                  "vision/picDevices/CIPC/picFaceplateCIPC_Status.pnl|INTERLOCK|AccessOperator|||",
                  "vision/unAlertPanel/unAlertScreen.pnl||AccessOperator|||",
                  "vision/unEventList/unEventList.pnl||AccessOperator|||",
                  "vision/unEventList/unEventList_SelectFilter.pnl|INTERLOCK,UNICOS,QPS,LHCOP|AccessOperator|||",
                  "vision/unObjectList/unObjectList.pnl||AccessOperator|||",
                  "fwTrending/fwTrendingPlotConfPanel.pnl|INTERLOCK,UNICOS|AccessOperator|||",
                  "fwTrending/fwTrendingPlotsPage.pnl|INTERLOCK,UNICOS|AccessOperator|||",
                  "fwTrending/fwTrendingPage.pnl|INTERLOCK,UNICOS|AccessOperator|||",
                  "fwTrending/fwTrendingHistogramConfig.pnl|INTERLOCK,UNICOS|AccessOperator|||",
                  "fwTrending/fwTrendingTrend.pnl|INTERLOCK,UNICOS,QPS|AccessOperator|||",
                  "fwTrending/fwTrendingHistogram.pnl|INTERLOCK,UNICOS|AccessOperator|||",
                  "vision/windowTree/unWindowTreeConfiguration.pnl|INTERLOCK,UNICOS|||AccessAdmin|",
                  "vision/trendTree/unTrendTreeConfiguration.pnl|INTERLOCK,UNICOS||AccessExpert||",
                  "vision/graphicalFrame/unicosSystemDPs.pnl|INTERLOCK,UNICOS||AccessExpert||",
                  "vision/graphicalFrame/unTreeDeleteDps.pnl|INTERLOCK,UNICOS|||AccessAdmin|",
                  "vision/unLHCLogging/unLHCLoggingStatus.pnl|INTERLOCK,UNICOS|AccessOperator|||",
                  "vision/unLaser/unLaserStatus.pnl|INTERLOCK,UNICOS|AccessOperator|||",
                  "vision/unCMWClient/unCMWClientStatus.pnl|INTERLOCK,UNICOS|AccessOperator|||",
                  "vision/unCMWServer/unCMWServerStatus.pnl|INTERLOCK,UNICOS|AccessOperator|||",
                  "vision/unicosObject/General/unModifyConfig.pnl|||AccessExpert||",
                  "vision/graphicalFrame/applicationConfiguration.pnl||||AccessAdmin|",
                  "vision/graphicalFrame/applicationConfigurationSetDefaultSettings.pnl||||AccessAdmin|",
                  "vision/graphicalFrame/addSystemIntegrity.pnl||||AccessAdmin|",
                  "vision/systemIntegrity/systemIntegrityConfiguration.pnl||||AccessAdmin|",
                  "vision/systemIntegrity/unSystemIntegrityConfigurationHMI.pnl||||AccessAdmin|",
                  "vision/systemIntegrity/unConfigMail.pnl||||AccessAdmin|",
                  "vision/systemIntegrity/stopDriverAndCtrl.pnl|INTERLOCK,UNICOS|||AccessAdmin|",
                  "vision/unEventList/unEventList_Configure.pnl||||AccessAdmin|",
                  "vision/unAlertPanel/unAlertPanel_ConfigureNames.pnl||||AccessAdmin|",
                  "vision/unicosObject/General/unObjectConfigurationHMI.pnl||||AccessAdmin|",
                  "vision/unicosObject/General/unObjectTrendConfiguration.pnl||||AccessAdmin|",
                  "vision/unLHCLogging/unLHCLoggingManager.pnl||||AccessAdmin|",
                  "vision/unLHCLogging/unLHCLoggingConfiguration.pnl||||AccessAdmin|",
                  "vision/unLHCLogging/unLHCLoggingConfigFile.pnl||||AccessAdmin|",
                  "vision/unLHCLogging/unLHCLoggingConfigFile_Check.pnl||||AccessAdmin|",
                  "vision/unLHCLogging/unLHCLoggingManagers_ExpertSetting.pnl||||AccessAdmin|",
                  "vision/unLHCLogging/unLHCLoggingArchives_ExpertSetting.pnl||||AccessAdmin|",
                  "vision/unLaser/unLaserManager.pnl||||AccessAdmin|",
                  "vision/unLaser/unLaserConfiguration.pnl||||AccessAdmin|",
                  "vision/unLaser/unLaserConfigFile.pnl||||AccessAdmin|",
                  "vision/unLaser/unLaserConfigFile_Check.pnl||||AccessAdmin|",
                  "vision/unLaser/unLaserExpertSetting.pnl||||AccessAdmin|",
                  "vision/unCMWClient/unCMWClientDriver.pnl||||AccessAdmin|",
                  "vision/unCMWServer/unCMWServerManager.pnl||||AccessAdmin|",
                  "vision/unCMWServer/unCMWServerConfiguration.pnl||||AccessAdmin|",
                  "vision/unCMWServer/unCMWServerConfigFile.pnl||||AccessAdmin|",
                  "vision/unCMWServer/unCMWServerConfigFile_Check.pnl||||AccessAdmin|",
                  "vision/unCMWServer/unCMWServerExpertSetting.pnl||||AccessAdmin|",
                  "vision/unicosObject/General/unFrontEndConfiguration.pnl||||AccessAdmin|",
                  "vision/distributedControl/unDistributedControl_configuration.pnl||||AccessAdmin|",
                  "vision/unicosObject/textLabel/unTextLabel_modify.pnl||||AccessAdmin|",
                  "vision/unicosObject/General/unOptionModes_Configure.pnl||||AccessAdmin|",
                  "vision/unGeneration/unGenerateDatabase.pnl|INTERLOCK,UNICOS|||AccessAdmin|",
                  "vision/unGeneration/unImportTree.pnl||||AccessAdmin|",
                  "vision/graphicalFrame/unFileAccessControlConfiguration.pnl||||AccessAdmin|",
                  "vision/graphicalFrame/unMenuConfiguration.pnl||||AccessAdmin|",
                  "vision/unDiagnostics/unArchiveStatistics.pnl||||AccessAdmin|",
                  "vision/unDiagnostics/unUIOverview.pnl||AccessOperator|||",
                  "vision/HDS_INTERLOCK/LHC_CIRCUIT_HDS_INTERLOCK_DisableCircuit.pnl|QPS|AccessAdmin|||",
                  "vision/HDS_INTERLOCK/LHC_CIRCUIT_HDS_INTERLOCK_SetDeviceTypeConfig.pnl|QPS|AccessAdmin|||",
                  "vision/HDS_INTERLOCK/LHC_CIRCUIT_HDS_INTERLOCK_Excluded_HDS.pnl|QPS|AccessAdmin|||",
                  "vision/systemIntegrity/unMailCategoryConfig.pnl|UNICOS,QPS|AccessAdmin|||",
				  "vision/unicosObject/General/unTypeConfigurator.pnl|INTERLOCK,UNICOS|||AccessAdmin|");
}

//@}
