/**@file

// unLHCCircuit_PIC.ctl
This library contains the functions to manage the CIPC used in the subsector widget in the LHCCircuit.

@par Creation Date
  24/09/2007

@par Modification History
  26/04/2011: Herve
  - CIR-23: Powering sub-sector and circuit widget: use PIC superlock for the animation

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui

@author 
  Frederic BERNARD (AB/CO-IS)
*/

/**
* @name LHCCircuit PIC functions
This is a set of functions provided to manage PIC data in LHCCircuit
* @{
*/

//------------------------------------------------------------------------------------------------------------------------
//		PIC CB FUNCTIONS
//------------------------------------------------------------------------------------------------------------------------
/** CIPC_widgetCB_1_1
*/

void unLHCCircuit_CIPC_widgetCB_1_1(	string sDpFEAl, int iFEAlValue, string sDpFEEnabled, bool bFEEnabled,
																												string sDevice1, bool bAux, string sDevice2, bool bEss, string sDevice3, bool bAbort, 
																												string sDevice4, bool bConnect, string sDevice5, bool bError,
                                      string sSuperLockDPE1, dyn_string dsComment1)
{
  synchronized(g_mFault)
  {
    g_bSuperLock_init = true;
    g_dsCIPCSuperLock = dsComment1;
  }
//DebugTN(g_dsCIPCSuperLock);
	//DebugN("unLHCCircuit_CIPC_widgetCB_1_1");
	unLHCCircuit_PIC_CIPC_Status(sDevice1, bAux & bEss & bAbort, bConnect & bError, iFEAlValue, bFEEnabled);
}

//------------------------------------------------------------------------------------------------------------------------
/** CIPC_widgetCB_2_2
*/

void unLHCCircuit_CIPC_widgetCB_2_2(	string sDpFEAl1, int iFEAlValue1, string sDpFEEnabled1, bool bFEEnabled1,
																												string sDpFEAl2, int iFEAlValue2, string sDpFEEnabled2, bool bFEEnabled2,
																												string sDevice1, bool bAux1, string sDevice2, bool bEss1, string sDevice3, bool bAbort1, 
																												string sDevice4, bool bConnect1, string sDevice5, bool bError1,
																												string sDevice6, bool bAux2, string sDevice7, bool bEss2, string sDevice8, bool bAbort2, 
																												string sDevice9, bool bConnect2, string sDevice10, bool bError2,
                                        string sSuperLockDPE1, dyn_string dsComment1, string sSuperLockDPE2, dyn_string dsComment2)
{
dyn_int diFEValue;

  synchronized(g_mFault)
  {
    g_bSuperLock_init = true;
    g_dsCIPCSuperLock = dsComment1;
    dynAppend(g_dsCIPCSuperLock, dsComment2);
  }
//DebugTN(g_dsCIPCSuperLock);
	//DebugN("unLHCCircuit_CIPC_widgetCB_2_2");
	diFEValue=makeDynInt(iFEAlValue1, iFEAlValue2);

	unLHCCircuit_PIC_CIPC_Status(		sDevice1,
																				(bAux1 & bAux2 & bEss1 & bEss2 & bAbort1 & bAbort2),
																				(bConnect1 & bConnect2 & bError1 & bError2),
																				dynMax(diFEValue),
																				bFEEnabled1 & bFEEnabled2);
}

//------------------------------------------------------------------------------------------------------------------------
/** CIPC_widgetCB_104_1
*/

void unLHCCircuit_CIPC_widgetCB_104_1(	string sDpFEAl, int iFEAlValue, string sDpFEEnabled, bool bFEEnabled,
																										string sDp1, bool bVal1, string sDp2, bool bVal2,
																										string sDpM1, bool bConnect, string sDpM2, bool bError)
{
	//DebugN("unLHCCircuit_CIPC_widgetCB_104_1");
	unLHCCircuit_PIC_CIPC_Status(sDp1, bVal1 & bVal2, bConnect & bError, iFEAlValue, bFEEnabled);
}

//-----------------------------------------------------------------------------------------------------------------------
/** CIPC_widgetCB_106_1
*/

void unLHCCircuit_CIPC_widgetCB_106_1(	string sDpFEAl, int iFEAlValue, string sDpFEEnabled, bool bFEEnabled,
																										string sDp1, bool bVal1, string sDp2, bool bVal2, string sDp3, bool bVal3,
																										string sDp4, bool bVal4,
																										string sDpM1, bool bConnect, string sDpM2, bool bError)
{
	//DebugN("unLHCCircuit_CIPC_widgetCB_106_1");
	unLHCCircuit_PIC_CIPC_Status(sDp1, bVal1 & bVal2 & bVal3 & bVal4, bConnect & bError, iFEAlValue, bFEEnabled);
}

//-----------------------------------------------------------------------------------------------------------------------
/** CIPC_widgetCB_107_1
*/

void unLHCCircuit_CIPC_widgetCB_107_1(	string sDpFEAl, int iFEAlValue, string sDpFEEnabled, bool bFEEnabled,
																										string sDp1, bool bVal1, string sDp2, bool bVal2, string sDp3, bool bVal3,
																										string sDp4, bool bVal4, string sDp5, bool bVal5,
																										string sDpM1, bool bConnect, string sDpM2, bool bError)
{
	//DebugN("unLHCCircuit_CIPC_widgetCB_107_1");	
	unLHCCircuit_PIC_CIPC_Status(sDp1, bVal1 & bVal2 & bVal3 & bVal4 & bVal5, bConnect & bError, iFEAlValue, bFEEnabled);
}

//-----------------------------------------------------------------------------------------------------------------------
/** CIPC_widgetCB_108_1
*/

void unLHCCircuit_CIPC_widgetCB_108_1(	string sDpFEAl, int iFEAlValue, string sDpFEEnabled, bool bFEEnabled,
																										string sDp1, bool bVal1, string sDp2, bool bVal2, string sDp3, bool bVal3,
																										string sDp4, bool bVal4, string sDp5, bool bVal5, string sDp6, bool bVal6,
																										string sDpM1, bool bConnect, string sDpM2, bool bError)
{
	//DebugN("unLHCCircuit_CIPC_widgetCB_108_1");	
	unLHCCircuit_PIC_CIPC_Status(sDp1, bVal1 & bVal2 & bVal3 & bVal4 & bVal5 & bVal6, bConnect & bError, iFEAlValue, bFEEnabled);
}

//-----------------------------------------------------------------------------------------------------------------------
/** CIPC_widgetCB_108_2
*/

void unLHCCircuit_CIPC_widgetCB_108_2(	string sDpFEAl1, int iFEAlValue1, string sDpFEEnabled1, bool bFEEnabled1,
																				string sDpFEAl2, int iFEAlValue2, string sDpFEEnabled2, bool bFEEnabled2,
																				string sDp1, bool bVal1, string sDp2, bool bVal2, string sDp3, bool bVal3,
																				string sDp4, bool bVal4, string sDp5, bool bVal5, string sDp6, bool bVal6,
																				string sDpM1, bool bConnect, string sDpM2, bool bError)
{
dyn_int diFEValue;

	//DebugN("unLHCCircuit_CIPC_widgetCB_108_2", sDpFEAl1, sDpFEAl2, sDp1, sDp2, sDp3, sDp4, sDp5, sDp6, sDpM1, sDpM2);	
	diFEValue=makeDynInt(sDpFEAl1, sDpFEAl2);
	unLHCCircuit_PIC_CIPC_Status(sDp1, bVal1 & bVal2 & bVal3 & bVal4 & bVal5 & bVal6, bConnect & bError, dynMax(diFEValue), bFEEnabled1 & bFEEnabled2);
}

//------------------------------------------------------------------------------------------------------------------------
/** PIC_CIPC_Status
*/

void unLHCCircuit_PIC_CIPC_Status(string sDevice, bool bBody, bool bFault, int iFEAlValue, bool bFEEnabled)
{
string sSystem;
bit32 value;
	
	sSystem=unGenericDpFunctions_getSystemName(sDevice);
  synchronized(g_mFault)
  {

	if (mappingHasKey(g_mFault, sSystem))
		{
		value=g_mFault[sSystem];
		setBit(value, BODY_ERROR, !bBody);	
		setBit(value, FAULT_CIPBAD, !bFault);
		setBit(value, FE_BAD, !((iFEAlValue<10) && bFEEnabled));
		//setBit(value, FAULT_FIPBAD, FALSE);
		g_mFault[sSystem]=value;
		}
	else
		DebugN("unLHCCircuit_PIC_CIPC_Status(): System UNKNOWN: " + sSystem);
unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, (string)txtwidget.text+" PIC", (string)txtwidget.text+" PIC", g_mFault, sDevice, bBody, bFault, iFEAlValue, bFEEnabled);   
  }
}

//------------------------------------------------------------------------------------------------------------------------
// unLHCCircuit_PIC_getCircuitsPICId
/** get PIC Dpe for the Circuit
*/

void unLHCCircuit_PIC_getCircuitsPICId(const string sSystem, const string sDpN, string sCircuit, const string sInterlockType, dyn_string &dsId)
{
dyn_string dsSignals;
string sDpe;
int i, iLen;

	if (patternMatch("RB.*", sCircuit))
		sCircuit=sCircuit+".EVEN";
			
	switch (sInterlockType)
		{	
		case CIRCUIT_A:
			dynAppend(dsSignals, "/ST_FAILURE_PIC");
			dynAppend(dsSignals, "/ST_ABORT_PIC");
			dynAppend(dsSignals, "/ST_DISCHARGE_PIC");
			dynAppend(dsSignals, "/CMD_PWR_PERM_PIC");
			dynAppend(dsSignals, "/CMD_ABORT_PIC");	
			dynAppend(dsSignals, "/CMD_DISCHARGE_PIC");
		break;

		case CIRCUIT_B1:
			dynAppend(dsSignals, "/ST_FAILURE_PIC");			
			dynAppend(dsSignals, "/ST_ABORT_PIC");
			dynAppend(dsSignals, "/CMD_PWR_PERM_PIC");
			dynAppend(dsSignals, "/CMD_ABORT_PIC");	
		break;

		case CIRCUIT_B2:
			dynAppend(dsSignals, "/ST_FAILURE_PIC");		
			dynAppend(dsSignals, "/ST_ABORT_PIC");
			dynAppend(dsSignals, "/CMD_PWR_PERM_B1_PIC");
			dynAppend(dsSignals, "/CMD_PWR_PERM_B2_PIC");
			dynAppend(dsSignals, "/CMD_ABORT_PIC");	
		break;

		case CIRCUIT_C:
			dynAppend(dsSignals, "/ST_FAILURE_PIC");		
			dynAppend(dsSignals, "/CMD_PWR_PERM_PIC");
		break;

		default:
			DebugN("unLHCCircuit_PIC_getCircuitsPICId(): Type UNKNOWN - " + sInterlockType + " - " + sCircuit);
		}
			
	iLen=dynlen(dsSignals);
	for(i=1; i<=iLen; i++)
		{
		sDpe=unGenericDpFunctions_dpAliasToName(sSystem + sCircuit + dsSignals[i]);
		if (sDpe!="")
			dynAppend(dsId, sDpe);
		else
			DebugN("unLHCCircuit_PIC_getCircuitsPICId(): Error to get Dpe - " + sSystem + sCircuit + dsSignals[i]);
		}
	
	if (iLen>0)
		{
		dynAppend(dsId, sDpN+".Calculated.ALL_ST_CONNECT");
		dynAppend(dsId, sDpN+".Calculated.ALL_ST_ERROR_POWER");
		}		
	
	dynUnique(dsId);
}

//------------------------------------------------------------------------------------------------------------------------
/** PIC_getSynopticDollarParameters
*/
			
void unLHCCircuit_PIC_getSynopticDollarParameters(const string sType, const string sCircuit, const string sSystem, const string sDpN, const string sAlias, dyn_string &dsPIC)
{
dyn_string dsParameters;
string sDpe;
bool bWrong;
  
        //special case: for RB.* -> 2 CIPC (ODD and EVEN): alway manage with the EVEN side
    	if(patternMatch("RB.*", sCircuit))
          {  
          sDpe=unGenericDpFunctions_dpAliasToName(sSystem + sCircuit+".EVEN/ST_FAILURE_PIC");
          if(!patternMatch(sDpN + "*", sDpe)) bWrong=TRUE;
          }
        
        if(!bWrong)
          {      
          dynAppend(dsPIC, "$ALIAS:" + sAlias);
  	      dynAppend(dsPIC, "$DPN:" + sDpN);
          }
}	

//------------------------------------------------------------------------------------------------------------------------
/** PIC_getEmptySynopticDollarParameters
*/

void unLHCCircuit_PIC_getEmptySynopticDollarParameters(const string sCircuit, dyn_string &dsPIC)
{
	dynAppend(dsPIC, "$ALIAS:");
	dynAppend(dsPIC, "$DPN:");
}

//@}
