/**@name SCRIPT: unLHC_CIRCUITDeviceListUpdate.ctl

@author: Herve Milcent (AB-CO)

Creation Date: 11/09/2007

Modification History: 
  16/11/2011: Herve
  - CIR-29  PVSS  use the XML PVSS native in PVSS 3.8-SP2 instead of the fwXML component. 
  - CIR-35 PVSS LHCCircuit scripts: system name and component version added in the MessageText log

	18/12/2007: Herve
		- add write subsystem1, subsystem2 and FE characteristics (FE, FE DP, FE DP type, FE Application) to file
		
	24/09/2007: Herve
		- add MessageText call

version 1.0

Purpose: 
Update the device list file for the OWS client

Usage: Public

PVSS manager usage: CTRL (PVSS00CTRL)

Constraints:
	. global variable used in the include files
	. PVSS version: 3.0 
	. operating system: WXP and Linux
	. distributed system: yes
*/

// include files
#uses "CtrlTreeDeviceOverview"
#uses "unTreeDeviceOverview.ctl"
#uses "unFilterDpFunctions.ctl"
#uses "unDeviceListUpdate.ctl"
#uses "unGenericObject.ctl"
#uses "unLHCCircuit_TreeOverview.ctl"
#uses "CtrlLHCCircuit_TreeOverview"
#uses "unLHCCircuit_xml.ctl"
    
// global
dyn_string g_dsCryoSystems;
int g_iDocum;
mapping g_mSystemId;

//@{

main()
{
dyn_string exceptionInfo;
string xml_file = LHC_CIRCUIT_CRYO_DATA_FILE;
string xml_full_name = getPath(DATA_REL_PATH, xml_file);

  // read Cryo XML file and define mapping with System Id
  g_iDocum=unLHCCircuit_xml_init(xml_full_name);
  if(g_iDocum>=0)
    {
    unLHCCircuit_xml_getSystemMapping(g_iDocum, g_mSystemId); 
 	  unLHCCircuit_xml_close(g_iDocum);
    unMessageText_send("*", "*", getSystemName()+"LHC_CIRCUITDeviceListUpdate", "user", "*", "EXPERTINFO", "CRYO definition loaded from: " + LHC_CIRCUIT_CRYO_DATA_FILE, exceptionInfo);          
    }          
  else
    unMessageText_send("*", "*", getSystemName()+"LHC_CIRCUITDeviceListUpdate", "user", "*", "EXPERTINFO", "CRYO XML file not found in the data folder: " + LHC_CIRCUIT_CRYO_DATA_FILE, exceptionInfo);
  
  // get the list of UNICOS device type
	unGenericDpFunctions_getUnicosObjects(g_dsDeviceType);

	unMessageText_send("*", "*", getSystemName()+"LHC_CIRCUITDeviceListUpdate", "user", "*", "EXPERTINFO", 
				"LHC_CIRCUITDeviceListUpdate initialisation done" , exceptionInfo);
  // register to all system for any triggerUpdate and re-create the corresponding file
	unDeviceListUpdate_registerTriggerUpdate("unLHC_CIRCUITDeviceListUpdate_applicationTrigger");
                       
  // register to all system to get the device list and update the corresponding file
	unDeviceListUpdate_register("unLHC_CIRCUITDeviceListUpdate_registerCB");
                             
  delay(0, 10);
  unMessageText_send("*", "*", getSystemName()+"LHC_CIRCUITDeviceListUpdate", "user", "*", "INFO", "LHC_CIRCUITDeviceListUpdate "+unGenericDpFunctions_getComponentsVersion(makeDynString("unCore", "unLHC_Circuit"))+" loaded", exceptionInfo);
}

//--------------------------------------------------------------------------------------------------------------------------------
// unLHC_CIRCUITDeviceListUpdate_registerCB
/**
Purpose:
Distributed control register callback function: create the device list file

Parameters:
	@param sDpe: string, input, the distributed control DPE of a PVSS system
	@param bSystemConnected: bool, input, state of the system

Usage: Public

Constraints:
	. PVSS version: 3.0 
	. PVSS manager type: PVSS00CTRL
	. operating system: WXP and Linux
	. distributed system: yes
*/
unLHC_CIRCUITDeviceListUpdate_registerCB(string sDpe, bool bSystemConnected)
{
	string sSystemName;
	string sDp = dpSubStr(sDpe, DPSUB_DP);
	bool bRemote, localConnected;
	dyn_dyn_string ddsTemp;
	
// get the system name and check if it is local or remote
	sSystemName = substr(sDp, strlen(c_unDistributedControl_dpName), strlen(sDp))+":";
	unDistributedControl_isRemote(bRemote, sSystemName);
        
//DebugN("unTreeDeviceOverview_register", g_m_bTreeDeviceOverviewRegisteredSystem, sDp, bSystemConnected, bRemote, sSystemName, mappingHasKey(g_m_bTreeDeviceOverviewRegisteredSystem, sSystemName));
	if(mappingHasKey(g_m_bTreeDeviceOverviewRegisteredSystem, sSystemName))
    {
		if(bRemote) localConnected = bSystemConnected;
		else localConnected = true;
                
		if(localConnected)
      {
    	// if the system is not already connected add it: trigger the display cb
    	// else do nothing
			if(!g_m_bTreeDeviceOverviewRegisteredSystem[sSystemName])
        {                                                                             
				g_m_bTreeDeviceOverviewRegisteredSystem[sSystemName] = true;
				g_m_bTreeDeviceOverviewSystemConnected[sSystemName] = true;
				g_m_ddsTreeDeviceOverviewDeviceList[sSystemName] = ddsTemp;

				if(!g_m_bTreeDeviceOverviewRegisteredSystemInitialized[sSystemName])
          { // startup, start thread
					g_m_bTreeDeviceOverview_gettingDeviceRunning[sSystemName] = true;
					startThread("unLHC_CIRCUITDeviceListUpdate_getAndWriteDevice", sSystemName, g_dsDeviceType);
				  }
				else
          { // not at startup
					if(!g_m_bTreeDeviceOverview_gettingDeviceRunning[sSystemName])
            {
						g_m_bTreeDeviceOverview_gettingDeviceRunning[sSystemName] = true;
						unLHC_CIRCUITDeviceListUpdate_getAndWriteDevice(sSystemName, g_dsDeviceType);
						g_m_bTreeDeviceOverview_gettingDeviceRunning[sSystemName] = false;
	  				}
		  		}                                                                
			  }
		  }
		else
      {
      // the system is disconnected
    	// if the system is connected delete it: trigger the display cb
    	// else do nothing
			if(g_m_bTreeDeviceOverviewRegisteredSystem[sSystemName])
        {
		  	g_m_bTreeDeviceOverviewRegisteredSystem[sSystemName] = false;
				g_m_bTreeDeviceOverviewSystemConnected[sSystemName] = false;
				g_m_ddsTreeDeviceOverviewDeviceList[sSystemName] = ddsTemp;
        //DebugN("set disc", sSystemName);
			  }
                        
			g_m_bTreeDeviceOverviewRegisteredSystemInitialized[sSystemName] = true;
  		}
  	}
}

//--------------------------------------------------------------------------------------------------------------------------------
// unLHC_CIRCUITDeviceListUpdate_applicationTrigger
/**
Purpose:
triggerUpdate CB function: update the device data list file

Parameters:
	@param sdpe: string, input, the dpe name
	@param bTriggerUpdate: bool, input, triggerUpdate value

Usage: Public

Constraints:
	. PVSS version: 3.0 
	. PVSS manager type: PVSS00CTRL, PVSS00NG and PVSS00NV
	. operating system: WXP and Linux
	. distributed system: yes
*/
unLHC_CIRCUITDeviceListUpdate_applicationTrigger(string sdpe, bool bTriggerUpdate)
{
	string sSystemName = unGenericDpFunctions_getSystemName(sdpe);

//DebugTN("start unLHC_CIRCUITDeviceListUpdate_applicationTrigger", sSystemName, g_m_bTreeDeviceOverviewRegisteredSystemInitialized[sSystemName], g_m_bTreeDeviceOverview_gettingDeviceRunning[sSystemName]);
	g_m_unicosGraphicalFrame_bUpdate[sSystemName] = true;
	if(g_m_bTreeDeviceOverviewRegisteredSystemInitialized[sSystemName]) {
		if(!g_m_bTreeDeviceOverview_gettingDeviceRunning[sSystemName]) {
//DebugN("trigger upda", sSystemName);
			g_m_bTreeDeviceOverview_gettingDeviceRunning[sSystemName] = true;
			unLHC_CIRCUITDeviceListUpdate_getAndWriteDevice(sSystemName, g_dsDeviceType, false);
			g_m_bTreeDeviceOverview_gettingDeviceRunning[sSystemName] = false;
		}
	}
//DebugTN("end unLHC_CIRCUITDeviceListUpdate_applicationTrigger", sdpe);
}

//--------------------------------------------------------------------------------------------------------------------------------
// unLHC_CIRCUITDeviceListUpdate_getAndWriteDevice
/**
Purpose:
Get and write the device list into a file

Parameters:
	@param sSystemName: string, input, the system name
	@param dsDeviceType: dyn_string, input, the list of device type

Usage: Public

Constraints:
	. PVSS version: 3.0 
	. PVSS manager type: PVSS00CTRL, PVSS00NG and PVSS00NV
	. operating system: WXP and Linux
	. distributed system: yes
*/
unLHC_CIRCUITDeviceListUpdate_getAndWriteDevice(string sSystemName, dyn_string dsDeviceType, bool bAlwaysWrite=true)
{
	dyn_dyn_string ddsTemp, ddsRead;
	bool bResult, bWrite=bAlwaysWrite;
	dyn_string dsPWS, dsCircuit, dsInterlockType, dsCircuitType, dsBuilding;
	int iDeviceNumber;
	dyn_dyn_string ddsCircuitTemp;
	
//DebugTN("*** start unLHC_CIRCUITDeviceListUpdate_getAndWriteDevice", sSystemName, bAlwaysWrite);
        
	g_m_bTreeDeviceOverview_deviceListAvailable[sSystemName] = false;
        
  //check if it is a CRYO system:
  //if yes, retrieve DEvice information from the XML file
  //else, normal way from distributed...
  g_dsCryoSystems=makeDynString("P18_12:","P2_23:","P4_34:","P4_45:","P6_56:","P6_67:","P8_78:","P8_81:");
  if(dynContains(g_dsCryoSystems, sSystemName)>0)  unLHCCircuit_TreeOverview_getCryoDeviceList(sSystemName, ddsTemp);
  else _unTreeDeviceOverview_getDeviceList(sSystemName, dsDeviceType, ddsTemp);
			
	if(!g_m_bTreeDeviceOverviewRegisteredSystemInitialized[sSystemName])
		bWrite = true;
        
	if(!bWrite)
    {
		_unTreeDeviceOverview_getDeviceListFomFile(sSystemName, ddsRead);
		if(!unDeviceListUpdate_compareDeviceList(ddsRead, ddsTemp)) 
			bWrite = true;
  	}

	if(bWrite)
	  {
		unDeviceListUpdate_writeFile(sSystemName, ddsTemp, "LHC_CIRCUITDeviceListUpdate", "LHC_CIRCUITDeviceListUpdate writing tree device file for ");
                
		_unLHCCircuit_TreeOverview_getCircuitDeviceProperty(sSystemName, ddsTemp, dsPWS, dsCircuit, dsInterlockType, dsCircuitType, dsBuilding, iDeviceNumber);
                
		ddsCircuitTemp[LHC_CIRCUIT_CIRCUIT_FILE_CIRCUIT] = dsCircuit;
		ddsCircuitTemp[LHC_CIRCUIT_CIRCUIT_FILE_INTERLOCKTYPE] = dsInterlockType;
		ddsCircuitTemp[LHC_CIRCUIT_CIRCUIT_FILE_CIRCUITTYPE] = dsCircuitType;
		ddsCircuitTemp[LHC_CIRCUIT_CIRCUIT_FILE_BUILDING] = dsBuilding;
		ddsCircuitTemp[LHC_CIRCUIT_CIRCUIT_FILE_PWS] = dsPWS;
		ddsCircuitTemp[LHC_CIRCUIT_CIRCUIT_FILE_NB_DEVICE] = iDeviceNumber;
                  
		g_m_ddsTreeDeviceOverviewDeviceList[sSystemName] = ddsTemp;
		unDeviceListUpdate_writeFile(sSystemName, ddsTemp, "LHC_CIRCUITDeviceListUpdate", "LHC_CIRCUITDeviceListUpdate writing tree device/circuit file for ", LHC_CIRCUIT_DEVICE_FILE);
		unDeviceListUpdate_writeFile(sSystemName, ddsCircuitTemp, "LHC_CIRCUITDeviceListUpdate", "LHC_CIRCUITDeviceListUpdate writing PWS circuit file for ", LHC_CIRCUIT_CIRCUIT_FILE);
		unDeviceListUpdate_writeSubsystemAndFECharacteristicsToFile(sSystemName, "LHC_CIRCUITDeviceListUpdate", "LHC_CIRCUITDeviceListUpdate writing sybsystems file for ", "LHC_CIRCUITDeviceListUpdate writing FE characteristics file for ");
		
    unDeviceListUpdate_triggerHMI(DEVUN_ADD_COMMAND, sSystemName);
	  }
	
	g_m_bTreeDeviceOverviewRegisteredSystemInitialized[sSystemName] = true;
	g_m_bTreeDeviceOverview_deviceListAvailable[sSystemName] = true;
	g_m_bTreeDeviceOverview_gettingDeviceRunning[sSystemName] = false;
//DebugTN("*** end unLHC_CIRCUITDeviceListUpdate_getAndWriteDevice", sSystemName, bWrite);
        
}

//-------------------------------------------------------------------------------------------------------------------------

//@}
